searchNodes=[{"doc":"A Rose Tree with Functional Zipper in Elixir What's a Rose Tree? A Rose Tree , also known as a multi-way or m-way tree by some, is a general-purpose, recursively defined data structure where each position can have an an arbitrary value and an arbitrary number of children. Each child is, itself, another Rose Tree. ExRoseTree is implemented as a very simple struct defstruct ~w(term children)a with an equally simple typespec: @type t ( ) :: % __MODULE__ { term : term ( ) , children : [ t ( ) ] } What's it good for? Practically speaking, a few good use cases for Rose Trees could be: outlines file systems parsing HTML/XML documents abstract syntax trees decision trees data visualization (org charts, family trees, taxonomies, nested menus) This implementation also comes with a companion ExRoseTree.Zipper data structure, and greatly enhances the usefulness of the standard Rose Tree. So what's a Zipper? A Zipper of a given data structure can be thought of as taking the derivative of that data structure. It provides an efficient, context-aware approach to traversing and manipulating the contents of the Rose Tree. In his foundational paper formalizing the idea, Gerard Huet perhaps describes it best: The basic idea is simple: the tree is turned inside-out like a returned glove, pointers from the root to the current position being reversed in a path structure. The current location holds both the downward current subtree and the upward path. All navigation and modification primitives operate on the location structure. Going up and down in the structure is analogous to closing and opening a zipper in a piece of clothing, whence the name. And what's a Zipper of a Rose Tree good for? In practice, ExRoseTree.Zipper can be used as an effective means of representing everything from a cursor in a text editor to a selected item in a nested sidebar/dropdown menu in a UI which needs to maintain persistent focus. Essentially, anything that has an arbitrary hierarchy and would necessitate or benefit from the capability of being context-aware could be a candidate for a Rose Tree with Zipper. Installation If available in Hex , the package can be installed by adding ex_rose_tree to your list of dependencies in mix.exs : def deps do [ { :ex_rose_tree , &quot;~&gt; 0.1.0&quot; } ] end Example Usage alias ExRoseTree , as : Tree alias ExRoseTree.Zipper Tree . new ( 1 ) # %ExRoseTree{term: 1, children: []} tree = Tree . new ( 1 , [ 2 , 3 , 4 , 5 ] ) # %ExRoseTree{term: 1, children: [ # %ExRoseTree{term: 2, children: []}, # %ExRoseTree{term: 3, children: []}, # %ExRoseTree{term: 4, children: []}, # %ExRoseTree{term: 5, children: []}, # ]} zipper = Zipper . new ( tree ) # %ExRoseTree.Zipper{ # focus: %ExRoseTree{ # term: 1, # children: [ # %ExRoseTree{term: 2, children: []}, # %ExRoseTree{term: 3, children: []}, # %ExRoseTree{term: 4, children: []}, # %ExRoseTree{term: 5, children: []} # ] # }, # prev: [], # next: [], # path: [] # } zipper = Zipper . last_child ( zipper ) # %ExRoseTree.Zipper{ # focus: %ExRoseTree{term: 5, children: []}, # prev: [ # %ExRoseTree{term: 4, children: []}, # %ExRoseTree{term: 3, children: []}, # %ExRoseTree{term: 2, children: []} # ], # next: [], # path: [%ExRoseTree.Zipper.Location{prev: [], term: 1, next: []}] # } zipper = Zipper . backward ( zipper ) # %ExRoseTree.Zipper{ # focus: %ExRoseTree{term: 4, children: []}, # prev: [ # %ExRoseTree{term: 3, children: []}, # %ExRoseTree{term: 2, children: []} # ], # next: [%ExRoseTree{term: 5, children: []}], # path: [%ExRoseTree.Zipper.Location{prev: [], term: 1, next: []}] # } zipper = Zipper . rewind_map ( zipper , &amp; Tree . map_term ( &amp;1 , fn t -&gt; t * 10 end ) ) # %ExRoseTree.Zipper{ # focus: %ExRoseTree{ # term: 10, # children: [ # %ExRoseTree{term: 2, children: []}, # %ExRoseTree{term: 3, children: []}, # %ExRoseTree{term: 40, children: []}, # %ExRoseTree{term: 5, children: []}, # ] # }, # prev: [], # next: [], # path: [] # } Zipper . to_tree ( zipper ) # %ExRoseTree{ # term: 10, # children: [ # %ExRoseTree{term: 2, children: []}, # %ExRoseTree{term: 3, children: []}, # %ExRoseTree{term: 40, children: []}, # %ExRoseTree{term: 5, children: []} # ] # } Testing and Disclaimer While great pains have been taken to provide extensive test coverage--over 800 tests at present, this library is still pre-1.0, so be sure to do your due diligence for your own use case. To run the test suite: $ mix deps.get $ mix test To run test coverage with excoveralls : $ mix deps.get $ mix coveralls or for HTML output: $ mix deps.get $ MIX_ENV=test mix coveralls.html","ref":"ExRoseTree.html","title":"ExRoseTree","type":"module"},{"doc":"Returns whether a list of values are all ExRoseTree s or not. Will return true if passed an empty list. Examples iex&gt; trees = for t &lt;- [ 5 , 4 , 3 , 2 , 1 ] , do : ExRoseTree . new ( t ) ...&gt; ExRoseTree . all_rose_trees? ( trees ) true","ref":"ExRoseTree.html#all_rose_trees?/1","title":"ExRoseTree.all_rose_trees?/1","type":"function"},{"doc":"Appends the given child to the tree 's children. Examples iex&gt; tree = ExRoseTree . new ( 5 , [ 4 , 3 , 2 , 1 ] ) ...&gt; ExRoseTree . append_child ( tree , 0 ) % ExRoseTree { term : 5 , children : [ % ExRoseTree { term : 4 , children : [ ] } , % ExRoseTree { term : 3 , children : [ ] } , % ExRoseTree { term : 2 , children : [ ] } , % ExRoseTree { term : 1 , children : [ ] } , % ExRoseTree { term : 0 , children : [ ] } ] }","ref":"ExRoseTree.html#append_child/2","title":"ExRoseTree.append_child/2","type":"function"},{"doc":"Initializes an empty tree. Examples iex&gt; ExRoseTree . empty ( ) % ExRoseTree { term : nil , children : [ ] }","ref":"ExRoseTree.html#empty/0","title":"ExRoseTree.empty/0","type":"function"},{"doc":"","ref":"ExRoseTree.html#empty?/1","title":"ExRoseTree.empty?/1","type":"macro"},{"doc":"Returns the children of an ExRoseTree . Examples iex&gt; tree = ExRoseTree . new ( 5 , [ 4 , 3 , 2 , 1 ] ) ...&gt; ExRoseTree . get_children ( tree ) [ % ExRoseTree { term : 4 , children : [ ] } , % ExRoseTree { term : 3 , children : [ ] } , % ExRoseTree { term : 2 , children : [ ] } , % ExRoseTree { term : 1 , children : [ ] } ]","ref":"ExRoseTree.html#get_children/1","title":"ExRoseTree.get_children/1","type":"function"},{"doc":"Returns the inner term of an ExRoseTree . Examples iex&gt; tree = ExRoseTree . new ( 5 ) ...&gt; ExRoseTree . get_term ( tree ) 5","ref":"ExRoseTree.html#get_term/1","title":"ExRoseTree.get_term/1","type":"function"},{"doc":"Returns whether or not the current tree has a child that matches the predicate . Examples iex&gt; tree = ExRoseTree . new ( 5 , [ 4 , 3 , 2 , 1 ] ) ...&gt; ExRoseTree . has_child? ( tree , &amp; ( &amp;1 . term == 2 ) ) true","ref":"ExRoseTree.html#has_child?/2","title":"ExRoseTree.has_child?/2","type":"function"},{"doc":"Inserts a new child into the tree 's children at the given index . Examples iex&gt; tree = ExRoseTree . new ( 5 , [ 4 , 3 , 2 , 1 ] ) ...&gt; ExRoseTree . insert_child ( tree , 3.5 , 2 ) % ExRoseTree { term : 5 , children : [ % ExRoseTree { term : 4 , children : [ ] } , % ExRoseTree { term : 3 , children : [ ] } , % ExRoseTree { term : 3.5 , children : [ ] } , % ExRoseTree { term : 2 , children : [ ] } , % ExRoseTree { term : 1 , children : [ ] } ] }","ref":"ExRoseTree.html#insert_child/3","title":"ExRoseTree.insert_child/3","type":"function"},{"doc":"","ref":"ExRoseTree.html#leaf?/1","title":"ExRoseTree.leaf?/1","type":"macro"},{"doc":"Applies the given function to each child tree. The map_fn should return a valid ExRoseTree struct. Examples iex&gt; tree = ExRoseTree . new ( 5 , [ 4 , 3 , 2 , 1 ] ) ...&gt; ExRoseTree . map_children ( tree , fn child -&gt; ...&gt; ExRoseTree . map_term ( child , fn x -&gt; x * 2 end ) ...&gt; end ) % ExRoseTree { term : 5 , children : [ % ExRoseTree { term : 8 , children : [ ] } , % ExRoseTree { term : 6 , children : [ ] } , % ExRoseTree { term : 4 , children : [ ] } , % ExRoseTree { term : 2 , children : [ ] } ] }","ref":"ExRoseTree.html#map_children/2","title":"ExRoseTree.map_children/2","type":"function"},{"doc":"Applies the given function to the tree term . Examples iex&gt; tree = ExRoseTree . new ( 5 ) ...&gt; ExRoseTree . map_term ( tree , fn x -&gt; x * 2 end ) % ExRoseTree { term : 10 , children : [ ] }","ref":"ExRoseTree.html#map_term/2","title":"ExRoseTree.map_term/2","type":"function"},{"doc":"Initializes a new tree with the given term . Examples iex&gt; ExRoseTree . new ( &quot;new term&quot; ) % ExRoseTree { term : &quot;new term&quot; , children : [ ] } iex&gt; ExRoseTree . new ( 5 , [ 4 , 3 , 2 , 1 ] ) % ExRoseTree { term : 5 , children : [ % ExRoseTree { term : 4 , children : [ ] } , % ExRoseTree { term : 3 , children : [ ] } , % ExRoseTree { term : 2 , children : [ ] } , % ExRoseTree { term : 1 , children : [ ] } ] } iex&gt; children = [ ...&gt; % ExRoseTree { term : 4 , children : [ ] } , ...&gt; 3 , ...&gt; % ExRoseTree { term : 2 , children : [ ] } , ...&gt; % ExRoseTree { term : 1 , children : [ ] } ...&gt; ] ...&gt; ExRoseTree . new ( 5 , children ) % ExRoseTree { term : 5 , children : [ % ExRoseTree { term : 4 , children : [ ] } , % ExRoseTree { term : 3 , children : [ ] } , % ExRoseTree { term : 2 , children : [ ] } , % ExRoseTree { term : 1 , children : [ ] } ] }","ref":"ExRoseTree.html#new/2","title":"ExRoseTree.new/2","type":"function"},{"doc":"","ref":"ExRoseTree.html#parent?/1","title":"ExRoseTree.parent?/1","type":"macro"},{"doc":"Removes the first child of the tree 's children. Examples iex&gt; tree = ExRoseTree . new ( 5 , [ 4 , 3 , 2 , 1 ] ) ...&gt; ExRoseTree . pop_first_child ( tree ) { % ExRoseTree { term : 5 , children : [ % ExRoseTree { term : 3 , children : [ ] } , % ExRoseTree { term : 2 , children : [ ] } , % ExRoseTree { term : 1 , children : [ ] } ] } , % ExRoseTree { term : 4 , children : [ ] } }","ref":"ExRoseTree.html#pop_first_child/1","title":"ExRoseTree.pop_first_child/1","type":"function"},{"doc":"Removes the last child of the tree 's children. Examples iex&gt; tree = ExRoseTree . new ( 5 , [ 4 , 3 , 2 , 1 ] ) ...&gt; ExRoseTree . pop_last_child ( tree ) { % ExRoseTree { term : 5 , children : [ % ExRoseTree { term : 4 , children : [ ] } , % ExRoseTree { term : 3 , children : [ ] } , % ExRoseTree { term : 2 , children : [ ] } ] } , % ExRoseTree { term : 1 , children : [ ] } }","ref":"ExRoseTree.html#pop_last_child/1","title":"ExRoseTree.pop_last_child/1","type":"function"},{"doc":"Prepends the given child to the tree 's children. Examples iex&gt; tree = ExRoseTree . new ( 5 , [ 4 , 3 , 2 , 1 ] ) ...&gt; ExRoseTree . prepend_child ( tree , 0 ) % ExRoseTree { term : 5 , children : [ % ExRoseTree { term : 0 , children : [ ] } , % ExRoseTree { term : 4 , children : [ ] } , % ExRoseTree { term : 3 , children : [ ] } , % ExRoseTree { term : 2 , children : [ ] } , % ExRoseTree { term : 1 , children : [ ] } ] }","ref":"ExRoseTree.html#prepend_child/2","title":"ExRoseTree.prepend_child/2","type":"function"},{"doc":"Removes a child from the tree 's children at the given index . Examples iex&gt; tree = ExRoseTree . new ( 5 , [ 4 , 3 , 2 , 1 ] ) ...&gt; ExRoseTree . remove_child ( tree , 2 ) { % ExRoseTree { term : 5 , children : [ % ExRoseTree { term : 4 , children : [ ] } , % ExRoseTree { term : 3 , children : [ ] } , % ExRoseTree { term : 1 , children : [ ] } ] } , % ExRoseTree { term : 2 , children : [ ] } }","ref":"ExRoseTree.html#remove_child/2","title":"ExRoseTree.remove_child/2","type":"function"},{"doc":"","ref":"ExRoseTree.html#rose_tree?/1","title":"ExRoseTree.rose_tree?/1","type":"macro"},{"doc":"Sets the tree children to the given list of children . Examples iex&gt; tree = ExRoseTree . new ( 5 ) ...&gt; ExRoseTree . set_children ( tree , [ 4 , 3 , 2 , 1 ] ) % ExRoseTree { term : 5 , children : [ % ExRoseTree { term : 4 , children : [ ] } , % ExRoseTree { term : 3 , children : [ ] } , % ExRoseTree { term : 2 , children : [ ] } , % ExRoseTree { term : 1 , children : [ ] } ] }","ref":"ExRoseTree.html#set_children/2","title":"ExRoseTree.set_children/2","type":"function"},{"doc":"Sets the tree term to the given term . Examples iex&gt; tree = ExRoseTree . new ( 5 ) ...&gt; ExRoseTree . set_term ( tree , &quot;five&quot; ) % ExRoseTree { term : &quot;five&quot; , children : [ ] }","ref":"ExRoseTree.html#set_term/2","title":"ExRoseTree.set_term/2","type":"function"},{"doc":"Given a seed value and an unfold_fn() , generates a new rose tree. Examples iex&gt; unfolder = fn ...&gt; x when x &gt; 0 -&gt; { Integer . to_string ( x ) , Enum . to_list ( 0 .. x - 1 ) } ...&gt; x -&gt; { Integer . to_string ( x ) , [ ] } ...&gt; end ...&gt; ExRoseTree . unfold ( 3 , unfolder ) % ExRoseTree { term : &quot;3&quot; , children : [ % ExRoseTree { term : &quot;0&quot; , children : [ ] } , % ExRoseTree { term : &quot;1&quot; , children : [ % ExRoseTree { term : &quot;0&quot; , children : [ ] } ] } , % ExRoseTree { term : &quot;2&quot; , children : [ % ExRoseTree { term : &quot;0&quot; , children : [ ] } , % ExRoseTree { term : &quot;1&quot; , children : [ % ExRoseTree { term : &quot;0&quot; , children : [ ] } ] } ] } ] }","ref":"ExRoseTree.html#unfold/2","title":"ExRoseTree.unfold/2","type":"function"},{"doc":"","ref":"ExRoseTree.html#t:predicate/0","title":"ExRoseTree.predicate/0","type":"type"},{"doc":"The foundational, recursive data type of an ExRoseTree . term can by any valid Erlang term() children is a list of t()","ref":"ExRoseTree.html#t:t/0","title":"ExRoseTree.t/0","type":"type"},{"doc":"A function that takes a seed value and returns a new ExRoseTree and a list of new seeds to use for children. Care must be taken that you don't create a function that infinitely creates new seeds, in other words, the function should have a terminating base case.","ref":"ExRoseTree.html#t:unfold_fn/0","title":"ExRoseTree.unfold_fn/0","type":"type"},{"doc":"A context-aware zipper for advanced traversal and manipulation of an ExRoseTree . Accompanying the Zipper are a large number of both navigation primitives and more complex traversal functions built out of said primitives. An attempt has been made at providing semantically meaningful names for these primitives, drawing from gender-neutral, familial taxonomy (with a few liberties taken in creating neolisms to better suit the domain here), with the aim of establishing a sort of navigational pattern language . The words first , last , next , and previous are ubiquitous and commonly paired with the likes of child , sibling , pibling (non-binary form of aunt/uncle), nibling (non-binary form of niece/nephew), and cousin to label specific navigation primitives. Other, less common words used for more specialized navigations include ancestral , descendant , and extended . Care has been taken to make naming conventions reflect the expected operations as closely as possible, though there are a few cases where it might not be entirely obvious, particularly for some of the more specialized operations, so be sure to read the documentation closely and test for your use case when using a navigational function for the first time. Many of these functions take an optional predicate() function which can be used to perform a navigational function until said predicate is satisfied. For example, ExRoseTree.Zipper.first_sibling(zipper, &amp;(&amp;1.term == 5)) will search, starting from the 0-th (first) index, the list of siblings that occur before but not after the current context for the first occurrence of a sibling with a term value equal to 5 . If none are found, the context will not have been moved, and the function returns nil . Note, the predicate function will default to Util.always/1 , which always returns true . When using the default predicate (in essence, not using a predicate) with this example, ExRoseTree.Zipper.first_sibling(zipper) , the function will simply move the context to the first sibling of the initial context. If the are no previous siblings, it will return nil . In general, most of the navigation primitives take constant time, while mutation is done at the current position and is a local operation.","ref":"ExRoseTree.Zipper.html","title":"ExRoseTree.Zipper","type":"module"},{"doc":"Accumulates an additional value using the provided acc_fn() while traversing the Zipper using the provided move_fn() . Returns a tuple including the new Zipper context and the accumulated value.","ref":"ExRoseTree.Zipper.html#accumulate/4","title":"ExRoseTree.Zipper.accumulate/4","type":"function"},{"doc":"Appends a new sibling to the Zipper 's next siblings.","ref":"ExRoseTree.Zipper.html#append_last_sibling/2","title":"ExRoseTree.Zipper.append_last_sibling/2","type":"function"},{"doc":"Appends a new sibling to the Zipper 's prev siblings.","ref":"ExRoseTree.Zipper.html#append_previous_sibling/2","title":"ExRoseTree.Zipper.append_previous_sibling/2","type":"function"},{"doc":"Traverses back through the Zipper in a depth-first manner.","ref":"ExRoseTree.Zipper.html#ascend/1","title":"ExRoseTree.Zipper.ascend/1","type":"function"},{"doc":"Ascends the Zipper and accumulates an additional value using the provided acc_fn() . Returns a tuple including the new Zipper and the accumulated value.","ref":"ExRoseTree.Zipper.html#ascend_accumulate/3","title":"ExRoseTree.Zipper.ascend_accumulate/3","type":"function"},{"doc":"Searches for a predicate match by ascending the Zipper . If no match is found, returns nil .","ref":"ExRoseTree.Zipper.html#ascend_find/2","title":"ExRoseTree.Zipper.ascend_find/2","type":"function"},{"doc":"Repeats a call to ascend/1 by the given number of reps .","ref":"ExRoseTree.Zipper.html#ascend_for/2","title":"ExRoseTree.Zipper.ascend_for/2","type":"function"},{"doc":"Ascends the Zipper if the provided predicate function returns true when applied to the next focus. Otherwise, returns nil .","ref":"ExRoseTree.Zipper.html#ascend_if/2","title":"ExRoseTree.Zipper.ascend_if/2","type":"function"},{"doc":"Ascends the Zipper and maps the term at each node using the provided map_fn() . Returns the new Zipper with mapped values.","ref":"ExRoseTree.Zipper.html#ascend_map/2","title":"ExRoseTree.Zipper.ascend_map/2","type":"function"},{"doc":"Ascends the Zipper until the root has been reached. If the root has previous siblings, will move the the first sibling of the root.","ref":"ExRoseTree.Zipper.html#ascend_to_root/1","title":"ExRoseTree.Zipper.ascend_to_root/1","type":"function"},{"doc":"Ascends the Zipper continuously until the provided predicate function returns true when applied to the next focus. Otherwise, returns nil .","ref":"ExRoseTree.Zipper.html#ascend_until/2","title":"ExRoseTree.Zipper.ascend_until/2","type":"function"},{"doc":"Ascends the Zipper while the given predicate remains true . If no custom predicate is given, ascend/1 will repeat until it no longer can.","ref":"ExRoseTree.Zipper.html#ascend_while/2","title":"ExRoseTree.Zipper.ascend_while/2","type":"function"},{"doc":"","ref":"ExRoseTree.Zipper.html#at_root?/1","title":"ExRoseTree.Zipper.at_root?/1","type":"macro"},{"doc":"Traverses backward through the Zipper in a breadth-first manner.","ref":"ExRoseTree.Zipper.html#backward/1","title":"ExRoseTree.Zipper.backward/1","type":"function"},{"doc":"Moves backward in the Zipper and accumulates an additional value using the provided acc_fn() . Returns a tuple including the new Zipper and the accumulated value.","ref":"ExRoseTree.Zipper.html#backward_accumulate/3","title":"ExRoseTree.Zipper.backward_accumulate/3","type":"function"},{"doc":"Searches for a predicate match by moving backward in the Zipper . If no match is found, returns nil .","ref":"ExRoseTree.Zipper.html#backward_find/2","title":"ExRoseTree.Zipper.backward_find/2","type":"function"},{"doc":"Repeats a call to backward/1 by the given number of reps .","ref":"ExRoseTree.Zipper.html#backward_for/2","title":"ExRoseTree.Zipper.backward_for/2","type":"function"},{"doc":"Moves backward in the Zipper if the provided predicate function returns true when applied to the next node. Otherwise, returns nil .","ref":"ExRoseTree.Zipper.html#backward_if/2","title":"ExRoseTree.Zipper.backward_if/2","type":"function"},{"doc":"Moves backward in the Zipper and maps the term at each node using the provided map_fn() . Returns the new Zipper with mapped values.","ref":"ExRoseTree.Zipper.html#backward_map/2","title":"ExRoseTree.Zipper.backward_map/2","type":"function"},{"doc":"Moves backward through the Zipper until the root has been reached. If the root has previous siblings, will move the the first sibling of the root.","ref":"ExRoseTree.Zipper.html#backward_to_root/1","title":"ExRoseTree.Zipper.backward_to_root/1","type":"function"},{"doc":"Moves backward in the Zipper continuously until the provided predicate function returns true when applied to the next node. Otherwise, returns nil .","ref":"ExRoseTree.Zipper.html#backward_until/2","title":"ExRoseTree.Zipper.backward_until/2","type":"function"},{"doc":"Moves backward in the Zipper while the given predicate remains true . If no custom predicate is given, backward/1 will repeat until it no longer can.","ref":"ExRoseTree.Zipper.html#backward_while/2","title":"ExRoseTree.Zipper.backward_while/2","type":"function"},{"doc":"Moves focus to the child at the specified index. If there are no children, or if the child does not exist at the index, returns nil . Examples iex&gt; tree = ExRoseTree . new ( 5 , [ 6 , 7 , 8 , 9 ] ) ...&gt; z = ExRoseTree.Zipper . new ( tree ) ...&gt; ExRoseTree.Zipper . child_at ( z , 2 ) % ExRoseTree.Zipper { focus : % ExRoseTree { term : 8 , children : [ ] } , prev : [ % ExRoseTree { term : 7 , children : [ ] } , % ExRoseTree { term : 6 , children : [ ] } ] , next : [ % ExRoseTree { term : 9 , children : [ ] } ] , path : [ % ExRoseTree.Zipper.Location { prev : [ ] , term : 5 , next : [ ] } ] }","ref":"ExRoseTree.Zipper.html#child_at/2","title":"ExRoseTree.Zipper.child_at/2","type":"function"},{"doc":"Returns the current focus of the Zipper . Examples iex&gt; tree = ExRoseTree . new ( 5 ) ...&gt; z = ExRoseTree.Zipper . new ( tree ) ...&gt; ExRoseTree.Zipper . current_focus ( z ) % ExRoseTree { term : 5 , children : [ ] }","ref":"ExRoseTree.Zipper.html#current_focus/1","title":"ExRoseTree.Zipper.current_focus/1","type":"function"},{"doc":"Returns the depth (as zero-based index) of the current focus. Examples iex&gt; loc_trees = for n &lt;- [ 4 , 3 , 2 , 1 ] , do : ExRoseTree . new ( n ) ...&gt; locs = for n &lt;- loc_trees , do : ExRoseTree.Zipper.Location . new ( n ) ...&gt; tree = ExRoseTree . new ( 5 ) ...&gt; z = ExRoseTree.Zipper . new ( tree , path : locs ) ...&gt; ExRoseTree.Zipper . depth_of_focus ( z ) 4","ref":"ExRoseTree.Zipper.html#depth_of_focus/1","title":"ExRoseTree.Zipper.depth_of_focus/1","type":"function"},{"doc":"Traverses forward through the Zipper in a depth-first manner.","ref":"ExRoseTree.Zipper.html#descend/1","title":"ExRoseTree.Zipper.descend/1","type":"function"},{"doc":"Descends the Zipper and accumulates an additional value using the provided acc_fn() . Returns a tuple including the new Zipper and the accumulated value.","ref":"ExRoseTree.Zipper.html#descend_accumulate/3","title":"ExRoseTree.Zipper.descend_accumulate/3","type":"function"},{"doc":"Searches for a predicate match by descending the Zipper . If no match is found, returns nil .","ref":"ExRoseTree.Zipper.html#descend_find/2","title":"ExRoseTree.Zipper.descend_find/2","type":"function"},{"doc":"Repeats a call to descend/1 by the given number of reps .","ref":"ExRoseTree.Zipper.html#descend_for/2","title":"ExRoseTree.Zipper.descend_for/2","type":"function"},{"doc":"Descends into the Zipper if the provided predicate function returns true when applied to the next focus. Otherwise, returns nil .","ref":"ExRoseTree.Zipper.html#descend_if/2","title":"ExRoseTree.Zipper.descend_if/2","type":"function"},{"doc":"Descends the Zipper and maps the term at each node using the provided map_fn() . Returns the new Zipper with mapped values.","ref":"ExRoseTree.Zipper.html#descend_map/2","title":"ExRoseTree.Zipper.descend_map/2","type":"function"},{"doc":"Descends the Zipper until the last node of the tree has been reached.","ref":"ExRoseTree.Zipper.html#descend_to_last/1","title":"ExRoseTree.Zipper.descend_to_last/1","type":"function"},{"doc":"Descends into the Zipper continuously until the provided predicate function returns true when applied to the next focus. Otherwise, returns nil .","ref":"ExRoseTree.Zipper.html#descend_until/2","title":"ExRoseTree.Zipper.descend_until/2","type":"function"},{"doc":"Descends the Zipper while the given predicate remains true . If no custom predicate is given, descend/1 will repeat until it no longer can.","ref":"ExRoseTree.Zipper.html#descend_while/2","title":"ExRoseTree.Zipper.descend_while/2","type":"function"},{"doc":"Returns an empty Zipper . Examples iex&gt; ExRoseTree.Zipper . empty ( ) % ExRoseTree.Zipper { focus : % ExRoseTree { term : nil , children : [ ] } , prev : [ ] , next : [ ] , path : [ ] }","ref":"ExRoseTree.Zipper.html#empty/0","title":"ExRoseTree.Zipper.empty/0","type":"function"},{"doc":"","ref":"ExRoseTree.Zipper.html#empty?/1","title":"ExRoseTree.Zipper.empty?/1","type":"macro"},{"doc":"Using the given move_fn() , searches for the first tree that satisfies the given predicate function.","ref":"ExRoseTree.Zipper.html#find/3","title":"ExRoseTree.Zipper.find/3","type":"function"},{"doc":"Recursively searches the path for the first, first &quot;ancestral&quot; pibling. That is, if a first pibling is not found for the parent, it will search the grandparent. If one is not found for the grandparent, it will search the great-grandparent. And so on, until it reaches the root. If the root is reached and it does not have a first pibling, the function returns nil .","ref":"ExRoseTree.Zipper.html#first_ancestral_pibling/2","title":"ExRoseTree.Zipper.first_ancestral_pibling/2","type":"function"},{"doc":"Moves focus to the first child. If there are no children, and this is a leaf, returns nil . Examples iex&gt; tree = ExRoseTree . new ( 5 , [ 6 , 7 , 8 , 9 ] ) ...&gt; z = ExRoseTree.Zipper . new ( tree ) ...&gt; ExRoseTree.Zipper . first_child ( z ) % ExRoseTree.Zipper { focus : % ExRoseTree { term : 6 , children : [ ] } , prev : [ ] , next : [ % ExRoseTree { term : 7 , children : [ ] } , % ExRoseTree { term : 8 , children : [ ] } , % ExRoseTree { term : 9 , children : [ ] } ] , path : [ % ExRoseTree.Zipper.Location { prev : [ ] , term : 5 , next : [ ] } ] } iex&gt; tree = ExRoseTree . new ( 5 , [ 6 , 7 , 8 , 9 ] ) ...&gt; z = ExRoseTree.Zipper . new ( tree ) ...&gt; ExRoseTree.Zipper . first_child ( z , fn x -&gt; x . term == 9 end ) % ExRoseTree.Zipper { focus : % ExRoseTree { term : 9 , children : [ ] } , prev : [ % ExRoseTree { term : 8 , children : [ ] } , % ExRoseTree { term : 7 , children : [ ] } , % ExRoseTree { term : 6 , children : [ ] } ] , next : [ ] , path : [ % ExRoseTree.Zipper.Location { prev : [ ] , term : 5 , next : [ ] } ] }","ref":"ExRoseTree.Zipper.html#first_child/2","title":"ExRoseTree.Zipper.first_child/2","type":"function"},{"doc":"Recursively searches the descendant branches of the first sibling for the first &quot;descendant nibling&quot; of the current focus. That is, if a first nibling is found, it will then look for the first child of that tree (the &quot;descendant nibling&quot;). It will repeat the search if one is found, and it will continue until no more are found, returning the last one visited.","ref":"ExRoseTree.Zipper.html#first_descendant_nibling/2","title":"ExRoseTree.Zipper.first_descendant_nibling/2","type":"function"},{"doc":"Searches for the first extended cousin or the first first-cousin of the focused tree. High level steps: Ascend path to find highest ExRoseTree.Zipper.Location with prev siblings. Starting with the first sibling, check each subtree from left to right, and if you reach the target depth and find a tree that satisifies any given predicate, stop there. Otherwise, continue left to right. If you return back to the starting ExRoseTree.Zipper.Location , descend the path to next deepest ExRoseTree.Zipper.Location and set as starting ExRoseTree.Zipper.Location . Goto step 2. If you return back to starting ExRoseTree.Zipper.Location , and it is also the ending ExRoseTree.Zipper.Location , and you have not found a suitable note at the right depth, you will not find one.","ref":"ExRoseTree.Zipper.html#first_extended_cousin/2","title":"ExRoseTree.Zipper.first_extended_cousin/2","type":"function"},{"doc":"Searches for the first child of the first extended cousin--aka, the first extended nibling--of the focused tree.","ref":"ExRoseTree.Zipper.html#first_extended_nibling/2","title":"ExRoseTree.Zipper.first_extended_nibling/2","type":"function"},{"doc":"Searches for the first extended cousin of the parent--aka, the first extended pibling--of the focused tree. Note: Extended Pibling here really means parent-cousin n-times removed, and is a bit of a neolism, since pibling technically means parent-sibling.","ref":"ExRoseTree.Zipper.html#first_extended_pibling/2","title":"ExRoseTree.Zipper.first_extended_pibling/2","type":"function"},{"doc":"Moves the focus to the first first-cousin -- the first child of the first pibling with children -- of the current focus. If not found, returns nil .","ref":"ExRoseTree.Zipper.html#first_first_cousin/2","title":"ExRoseTree.Zipper.first_first_cousin/2","type":"function"},{"doc":"Moves the focus to the first grandchild -- the first child of the first child -- of the focus. If there are no grandchildren, moves to the next sibling of the first child and looks for that tree's first child. This repeats until the first grandchild is found or it returns nil if none are found.","ref":"ExRoseTree.Zipper.html#first_grandchild/2","title":"ExRoseTree.Zipper.first_grandchild/2","type":"function"},{"doc":"Moves the focus to the first grandpibling -- the first sibling of the grandparent -- of the current focus. If not found, returns nil .","ref":"ExRoseTree.Zipper.html#first_grandpibling/2","title":"ExRoseTree.Zipper.first_grandpibling/2","type":"function"},{"doc":"Moves the focus to the first great-grandchild -- the first child of the first grandchild -- of the focus. If there are no great-grandchildren, moves to the next sibling of the first grandchild and looks for that tree's first child. This repeats until the first great-grandchild is found or it returns nil if none are found.","ref":"ExRoseTree.Zipper.html#first_great_grandchild/2","title":"ExRoseTree.Zipper.first_great_grandchild/2","type":"function"},{"doc":"Moves the focus to the first nibling -- the first child of the first sibling with children -- before the current focus. If not found, returns nil .","ref":"ExRoseTree.Zipper.html#first_nibling/2","title":"ExRoseTree.Zipper.first_nibling/2","type":"function"},{"doc":"Moves the focus to the first nibling for a specific sibling -- the first child of the sibling at the given index -- of the current focus. If not found, returns nil .","ref":"ExRoseTree.Zipper.html#first_nibling_at_sibling/3","title":"ExRoseTree.Zipper.first_nibling_at_sibling/3","type":"function"},{"doc":"Moves the focus to the first pibling -- the first sibling of the parent -- of the current focus. If not found, returns nil .","ref":"ExRoseTree.Zipper.html#first_pibling/2","title":"ExRoseTree.Zipper.first_pibling/2","type":"function"},{"doc":"Moves the focus to the first second-cousin -- the first grandchild of the first grandpibling with grandchildren -- of the current focus. If not found, returns nil .","ref":"ExRoseTree.Zipper.html#first_second_cousin/2","title":"ExRoseTree.Zipper.first_second_cousin/2","type":"function"},{"doc":"Moves focus to the first sibling from the current focus. If there are no more siblings before the current focus, returns nil . Examples iex&gt; prev = for n &lt;- [ 1 , 2 , 3 , 4 ] , do : ExRoseTree . new ( n ) ...&gt; next = for n &lt;- [ 6 , 7 , 8 , 9 ] , do : ExRoseTree . new ( n ) ...&gt; tree = ExRoseTree . new ( 5 ) ...&gt; z = ExRoseTree.Zipper . new ( tree , prev : prev , next : next ) ...&gt; ExRoseTree.Zipper . first_sibling ( z ) % ExRoseTree.Zipper { focus : % ExRoseTree { term : 1 , children : [ ] } , prev : [ ] , next : [ % ExRoseTree { term : 2 , children : [ ] } , % ExRoseTree { term : 3 , children : [ ] } , % ExRoseTree { term : 4 , children : [ ] } , % ExRoseTree { term : 5 , children : [ ] } , % ExRoseTree { term : 6 , children : [ ] } , % ExRoseTree { term : 7 , children : [ ] } , % ExRoseTree { term : 8 , children : [ ] } , % ExRoseTree { term : 9 , children : [ ] } ] , path : [ ] }","ref":"ExRoseTree.Zipper.html#first_sibling/2","title":"ExRoseTree.Zipper.first_sibling/2","type":"function"},{"doc":"Returns the children of the Zipper 's current focus. A shortcut to ExRoseTree.get_children/1 . Examples iex&gt; tree = ExRoseTree . new ( 5 , [ 1 , 2 , 3 , 4 ] ) ...&gt; z = ExRoseTree.Zipper . new ( tree ) ...&gt; ExRoseTree.Zipper . focused_children ( z ) [ % ExRoseTree { term : 1 , children : [ ] } , % ExRoseTree { term : 2 , children : [ ] } , % ExRoseTree { term : 3 , children : [ ] } , % ExRoseTree { term : 4 , children : [ ] } ]","ref":"ExRoseTree.Zipper.html#focused_children/1","title":"ExRoseTree.Zipper.focused_children/1","type":"function"},{"doc":"Returns the term of the current focus of the Zipper . A shortcut to using ExRoseTree.get_term/1 . Examples iex&gt; tree = ExRoseTree . new ( 5 ) ...&gt; z = ExRoseTree.Zipper . new ( tree ) ...&gt; ExRoseTree.Zipper . focused_term ( z ) 5","ref":"ExRoseTree.Zipper.html#focused_term/1","title":"ExRoseTree.Zipper.focused_term/1","type":"function"},{"doc":"Traverses forward through the Zipper in a breadth-first manner.","ref":"ExRoseTree.Zipper.html#forward/1","title":"ExRoseTree.Zipper.forward/1","type":"function"},{"doc":"Moves forward in the Zipper and accumulates an additional value using the provided acc_fn() . Returns a tuple including the new Zipper and the accumulated value.","ref":"ExRoseTree.Zipper.html#forward_accumulate/3","title":"ExRoseTree.Zipper.forward_accumulate/3","type":"function"},{"doc":"Searches for a predicate match by moving forward in the Zipper . If no match is found, returns nil .","ref":"ExRoseTree.Zipper.html#forward_find/2","title":"ExRoseTree.Zipper.forward_find/2","type":"function"},{"doc":"Repeats a call to forward/1 by the given number of reps .","ref":"ExRoseTree.Zipper.html#forward_for/2","title":"ExRoseTree.Zipper.forward_for/2","type":"function"},{"doc":"Moves forward in the Zipper if the provided predicate function returns true when applied to the next node. Otherwise, returns nil .","ref":"ExRoseTree.Zipper.html#forward_if/2","title":"ExRoseTree.Zipper.forward_if/2","type":"function"},{"doc":"Moves forward in the Zipper and maps the term at each node using the provided map_fn() . Returns the new Zipper with mapped values.","ref":"ExRoseTree.Zipper.html#forward_map/2","title":"ExRoseTree.Zipper.forward_map/2","type":"function"},{"doc":"Moves forward through the Zipper until the last node of the tree has been reached.","ref":"ExRoseTree.Zipper.html#forward_to_last/1","title":"ExRoseTree.Zipper.forward_to_last/1","type":"function"},{"doc":"Moves forward in the Zipper continuously until the provided predicate function returns true when applied to the next node. Otherwise, returns nil .","ref":"ExRoseTree.Zipper.html#forward_until/2","title":"ExRoseTree.Zipper.forward_until/2","type":"function"},{"doc":"Moves forward in the Zipper while the given predicate remains true . If no custom predicate is given, forward/1 will repeat until it no longer can.","ref":"ExRoseTree.Zipper.html#forward_while/2","title":"ExRoseTree.Zipper.forward_while/2","type":"function"},{"doc":"Builds a new Zipper from a list of ExRoseTree.Zipper.Location s. Examples iex&gt; locs = for loc &lt;- [ 3 , 2 , 1 ] , do : ExRoseTree.Zipper.Location . new ( loc ) ...&gt; ExRoseTree.Zipper . from_locations ( locs ) % ExRoseTree.Zipper { focus : % ExRoseTree { term : 3 , children : [ ] } , prev : [ ] , next : [ ] , path : [ % ExRoseTree.Zipper.Location { prev : [ ] , term : 2 , next : [ ] } , % ExRoseTree.Zipper.Location { prev : [ ] , term : 1 , next : [ ] } ] }","ref":"ExRoseTree.Zipper.html#from_locations/1","title":"ExRoseTree.Zipper.from_locations/1","type":"function"},{"doc":"Moves the focus to the grandparent -- the parent of the parent -- of the focus, if possible. If there is no grandparent, returns nil .","ref":"ExRoseTree.Zipper.html#grandparent/1","title":"ExRoseTree.Zipper.grandparent/1","type":"function"},{"doc":"Moves the focus to the great-grandparent -- parent of the grand-parent -- of the focus, if available. If there is no great-grandparent, returns nil .","ref":"ExRoseTree.Zipper.html#great_grandparent/1","title":"ExRoseTree.Zipper.great_grandparent/1","type":"function"},{"doc":"","ref":"ExRoseTree.Zipper.html#has_children?/1","title":"ExRoseTree.Zipper.has_children?/1","type":"macro"},{"doc":"","ref":"ExRoseTree.Zipper.html#has_siblings?/1","title":"ExRoseTree.Zipper.has_siblings?/1","type":"macro"},{"doc":"Returns the index (zero-based) of the current focus with respect to any potential siblings it may have. Examples iex&gt; prev = for n &lt;- [ 1 , 2 , 3 , 4 ] , do : ExRoseTree . new ( n ) ...&gt; tree = ExRoseTree . new ( 5 ) ...&gt; z = ExRoseTree.Zipper . new ( tree , prev : prev ) ...&gt; ExRoseTree.Zipper . index_of_focus ( z ) 4","ref":"ExRoseTree.Zipper.html#index_of_focus/1","title":"ExRoseTree.Zipper.index_of_focus/1","type":"function"},{"doc":"Returns the index (zero-based) of the current focus' grandparent with respect to any potentital siblings it may have. If the current focus has no grandparent, returns nil . Examples iex&gt; grandparent_siblings = for n &lt;- [ 3 , 2 , 1 ] , do : ExRoseTree . new ( n ) ...&gt; grandparent_loc = ExRoseTree.Zipper.Location . new ( 4 , prev : grandparent_siblings ) ...&gt; parent_loc = ExRoseTree.Zipper.Location . new ( 5 ) ...&gt; tree = ExRoseTree . new ( 6 ) ...&gt; z = ExRoseTree.Zipper . new ( tree , path : [ parent_loc , grandparent_loc ] ) ...&gt; ExRoseTree.Zipper . index_of_grandparent ( z ) 3","ref":"ExRoseTree.Zipper.html#index_of_grandparent/1","title":"ExRoseTree.Zipper.index_of_grandparent/1","type":"function"},{"doc":"Returns the index (zero-based) of the current focus' parent with respect to any potentital siblings it may have. If the current focus has no parent, returns nil . Examples iex&gt; parent_siblings = for n &lt;- [ 3 , 2 , 1 ] , do : ExRoseTree . new ( n ) ...&gt; parent_loc = ExRoseTree.Zipper.Location . new ( 4 , prev : parent_siblings ) ...&gt; tree = ExRoseTree . new ( 5 ) ...&gt; z = ExRoseTree.Zipper . new ( tree , path : [ parent_loc ] ) ...&gt; ExRoseTree.Zipper . index_of_parent ( z ) 3","ref":"ExRoseTree.Zipper.html#index_of_parent/1","title":"ExRoseTree.Zipper.index_of_parent/1","type":"function"},{"doc":"Inserts a new sibling in the Zipper 's next siblings at the given index.","ref":"ExRoseTree.Zipper.html#insert_next_sibling_at/3","title":"ExRoseTree.Zipper.insert_next_sibling_at/3","type":"function"},{"doc":"Inserts a new sibling in the Zipper 's prev siblings at the given index.","ref":"ExRoseTree.Zipper.html#insert_previous_sibling_at/3","title":"ExRoseTree.Zipper.insert_previous_sibling_at/3","type":"function"},{"doc":"Recursively searches the path for the first, last &quot;ancestral&quot; pibling. That is, if a last pibling is not found for the parent, it will search the grandparent. If one is not found for the grandparent, it will search the great-grandparent. And so on, until it reaches the root. If the root is reached and it does not have a last pibling, the function returns nil .","ref":"ExRoseTree.Zipper.html#last_ancestral_pibling/2","title":"ExRoseTree.Zipper.last_ancestral_pibling/2","type":"function"},{"doc":"Moves focus to the last child. If there are no children, and this is a leaf, returns nil . Examples iex&gt; tree = ExRoseTree . new ( 5 , [ 6 , 7 , 8 , 9 ] ) ...&gt; z = ExRoseTree.Zipper . new ( tree ) ...&gt; ExRoseTree.Zipper . last_child ( z ) % ExRoseTree.Zipper { focus : % ExRoseTree { term : 9 , children : [ ] } , prev : [ % ExRoseTree { term : 8 , children : [ ] } , % ExRoseTree { term : 7 , children : [ ] } , % ExRoseTree { term : 6 , children : [ ] } ] , next : [ ] , path : [ % ExRoseTree.Zipper.Location { prev : [ ] , term : 5 , next : [ ] } ] }","ref":"ExRoseTree.Zipper.html#last_child/2","title":"ExRoseTree.Zipper.last_child/2","type":"function"},{"doc":"Recursively searches the descendant branches of the last sibling for the last &quot;descendant nibling&quot; of the current focus. That is, if a last nibling is found, it will then look for the last child of that tree (the &quot;descendant nibling&quot;). It will repeat the search if one is found, and it will continue until no more are found, returning the last one visited.","ref":"ExRoseTree.Zipper.html#last_descendant_nibling/2","title":"ExRoseTree.Zipper.last_descendant_nibling/2","type":"function"},{"doc":"Searches for the last extended cousin or the last first-cousin of the focused tree.","ref":"ExRoseTree.Zipper.html#last_extended_cousin/2","title":"ExRoseTree.Zipper.last_extended_cousin/2","type":"function"},{"doc":"Searches for the last child of the last extended cousin--aka, the last extended nibling--of the focused tree.","ref":"ExRoseTree.Zipper.html#last_extended_nibling/2","title":"ExRoseTree.Zipper.last_extended_nibling/2","type":"function"},{"doc":"Searches for the last extended cousin of the parent--aka, the last extended pibling--of the focused tree. Note: Extended Pibling here really means parent-cousin n-times removed, and is a bit of a neolism, since pibling technically means parent-sibling.","ref":"ExRoseTree.Zipper.html#last_extended_pibling/2","title":"ExRoseTree.Zipper.last_extended_pibling/2","type":"function"},{"doc":"Moves the focus to the last first-cousin -- the last child of the last pibling with children -- of the current focus. If not found, returns nil .","ref":"ExRoseTree.Zipper.html#last_first_cousin/2","title":"ExRoseTree.Zipper.last_first_cousin/2","type":"function"},{"doc":"Moves the focus to the last grandchild -- the last child of the last child -- of the focus. If there are no grandchildren, moves to the previous sibling of the last child and looks for that tree's last child. This repeats until the first grandchild is found or it returns nil if none are found.","ref":"ExRoseTree.Zipper.html#last_grandchild/2","title":"ExRoseTree.Zipper.last_grandchild/2","type":"function"},{"doc":"Moves the focus to the last grandpibling -- the last sibling of the grandparent -- of the current focus. If not found, returns nil .","ref":"ExRoseTree.Zipper.html#last_grandpibling/2","title":"ExRoseTree.Zipper.last_grandpibling/2","type":"function"},{"doc":"Moves the focus to the last great-grandchild -- the last child of the last grandchild -- of the focus. If there are no great-grandchildren, moves to the previous sibling of the last grandchild and looks for that tree's last child. This repeats until the last great-grandchild is found or it returns nil if none are found.","ref":"ExRoseTree.Zipper.html#last_great_grandchild/2","title":"ExRoseTree.Zipper.last_great_grandchild/2","type":"function"},{"doc":"Moves the focus to the last nibling -- the last child of the last sibling with children -- before the current focus. If not found, returns nil .","ref":"ExRoseTree.Zipper.html#last_nibling/2","title":"ExRoseTree.Zipper.last_nibling/2","type":"function"},{"doc":"Moves the focus to the last nibling for a specific sibling -- the last child of the sibling at the given index -- of the current focus. If not found, returns nil .","ref":"ExRoseTree.Zipper.html#last_nibling_at_sibling/3","title":"ExRoseTree.Zipper.last_nibling_at_sibling/3","type":"function"},{"doc":"Moves the focus to the last pibling -- the last sibling of the parent -- of the current focus. If not found, returns nil .","ref":"ExRoseTree.Zipper.html#last_pibling/2","title":"ExRoseTree.Zipper.last_pibling/2","type":"function"},{"doc":"Moves the focus to the last second-cousin -- the last grandchild of the last grandpibling with grandchildren -- of the current focus. If not found, returns nil .","ref":"ExRoseTree.Zipper.html#last_second_cousin/2","title":"ExRoseTree.Zipper.last_second_cousin/2","type":"function"},{"doc":"Moves focus to the last sibling from the current focus. If there are no more siblings after the current focus, returns nil . Examples iex&gt; prev = for n &lt;- [ 1 , 2 , 3 , 4 ] , do : ExRoseTree . new ( n ) ...&gt; next = for n &lt;- [ 6 , 7 , 8 , 9 ] , do : ExRoseTree . new ( n ) ...&gt; tree = ExRoseTree . new ( 5 ) ...&gt; z = ExRoseTree.Zipper . new ( tree , prev : prev , next : next ) ...&gt; ExRoseTree.Zipper . last_sibling ( z ) % ExRoseTree.Zipper { focus : % ExRoseTree { term : 9 , children : [ ] } , prev : [ % ExRoseTree { term : 8 , children : [ ] } , % ExRoseTree { term : 7 , children : [ ] } , % ExRoseTree { term : 6 , children : [ ] } , % ExRoseTree { term : 5 , children : [ ] } , % ExRoseTree { term : 4 , children : [ ] } , % ExRoseTree { term : 3 , children : [ ] } , % ExRoseTree { term : 2 , children : [ ] } , % ExRoseTree { term : 1 , children : [ ] } ] , next : [ ] , path : [ ] }","ref":"ExRoseTree.Zipper.html#last_sibling/2","title":"ExRoseTree.Zipper.last_sibling/2","type":"function"},{"doc":"Descend the left-most edge until it can go no further or until the optional predicate matches. Does not include siblings of starting focus.","ref":"ExRoseTree.Zipper.html#leftmost_descendant/2","title":"ExRoseTree.Zipper.leftmost_descendant/2","type":"function"},{"doc":"Traverses the Zipper using the provided move_fn() and maps the term at each node using the provided map_fn() . Returns the new Zipper with mapped values.","ref":"ExRoseTree.Zipper.html#map/3","title":"ExRoseTree.Zipper.map/3","type":"function"},{"doc":"Applies the given function to the current focus. Examples iex&gt; tree = ExRoseTree . new ( 5 ) ...&gt; z = ExRoseTree.Zipper . new ( tree ) ...&gt; map_fn = &amp; ExRoseTree . set_children ( &amp;1 , [ 6 , 7 , 8 , 9 ] ) ...&gt; ExRoseTree.Zipper . map_focus ( z , &amp; map_fn . ( &amp;1 ) ) % ExRoseTree.Zipper { focus : % ExRoseTree { term : 5 , children : [ % ExRoseTree { term : 6 , children : [ ] } , % ExRoseTree { term : 7 , children : [ ] } , % ExRoseTree { term : 8 , children : [ ] } , % ExRoseTree { term : 9 , children : [ ] } , ] } , prev : [ ] , next : [ ] , path : [ ] }","ref":"ExRoseTree.Zipper.html#map_focus/2","title":"ExRoseTree.Zipper.map_focus/2","type":"function"},{"doc":"Applies the given function to the focused children of the current focus. A shortcut to ExRoseTree.map_children/2 and set_focus/2 . Examples iex&gt; tree = ExRoseTree . new ( 5 , [ 1 , 2 , 3 , 4 ] ) ...&gt; z = ExRoseTree.Zipper . new ( tree ) ...&gt; map_fn = &amp; ExRoseTree . map_term ( &amp;1 , fn x -&gt; x * 2 end ) ...&gt; ExRoseTree.Zipper . map_focused_children ( z , map_fn ) % ExRoseTree.Zipper { focus : % ExRoseTree { term : 5 , children : [ % ExRoseTree { term : 2 , children : [ ] } , % ExRoseTree { term : 4 , children : [ ] } , % ExRoseTree { term : 6 , children : [ ] } , % ExRoseTree { term : 8 , children : [ ] } ] } , prev : [ ] , next : [ ] , path : [ ] }","ref":"ExRoseTree.Zipper.html#map_focused_children/2","title":"ExRoseTree.Zipper.map_focused_children/2","type":"function"},{"doc":"Applies the given function to the term of the current focus. A shortcut to using ExRoseTree.map_term/2 with set_focus/2 . Examples iex&gt; tree = ExRoseTree . new ( 5 ) ...&gt; z = ExRoseTree.Zipper . new ( tree ) ...&gt; map_fn = fn term -&gt; term * 2 end ...&gt; ExRoseTree.Zipper . map_focused_term ( z , &amp; map_fn . ( &amp;1 ) ) % ExRoseTree.Zipper { focus : % ExRoseTree { term : 10 , children : [ ] } , prev : [ ] , next : [ ] , path : [ ] }","ref":"ExRoseTree.Zipper.html#map_focused_term/2","title":"ExRoseTree.Zipper.map_focused_term/2","type":"function"},{"doc":"Applies the given function to all next siblings of the current focus without moving the Zipper . Examples iex&gt; next = for n &lt;- [ 6 , 7 , 8 , 9 ] , do : ExRoseTree . new ( n ) ...&gt; tree = ExRoseTree . new ( 5 ) ...&gt; z = ExRoseTree.Zipper . new ( tree , next : next ) ...&gt; map_fn = &amp; ExRoseTree . map_term ( &amp;1 , fn term -&gt; term * 2 end ) ...&gt; ExRoseTree.Zipper . map_next_siblings ( z , &amp; map_fn . ( &amp;1 ) ) % ExRoseTree.Zipper { focus : % ExRoseTree { term : 5 , children : [ ] } , prev : [ ] , next : [ % ExRoseTree { term : 12 , children : [ ] } , % ExRoseTree { term : 14 , children : [ ] } , % ExRoseTree { term : 16 , children : [ ] } , % ExRoseTree { term : 18 , children : [ ] } ] , path : [ ] }","ref":"ExRoseTree.Zipper.html#map_next_siblings/2","title":"ExRoseTree.Zipper.map_next_siblings/2","type":"function"},{"doc":"Applies the given function to path of ExRoseTree.Zipper.Location s from the current focus back to the root without moving the Zipper . Examples iex&gt; path = for n &lt;- [ 4 , 3 , 2 , 1 ] , do : ExRoseTree.Zipper.Location . new ( n ) ...&gt; tree = ExRoseTree . new ( 5 ) ...&gt; z = ExRoseTree.Zipper . new ( tree , path : path ) ...&gt; map_fn = &amp; ExRoseTree.Zipper.Location . map_term ( &amp;1 , fn term -&gt; term * 2 end ) ...&gt; ExRoseTree.Zipper . map_path ( z , &amp; map_fn . ( &amp;1 ) ) % ExRoseTree.Zipper { focus : % ExRoseTree { term : 5 , children : [ ] } , prev : [ ] , next : [ ] , path : [ % ExRoseTree.Zipper.Location { prev : [ ] , term : 8 , next : [ ] } , % ExRoseTree.Zipper.Location { prev : [ ] , term : 6 , next : [ ] } , % ExRoseTree.Zipper.Location { prev : [ ] , term : 4 , next : [ ] } , % ExRoseTree.Zipper.Location { prev : [ ] , term : 2 , next : [ ] } ] }","ref":"ExRoseTree.Zipper.html#map_path/2","title":"ExRoseTree.Zipper.map_path/2","type":"function"},{"doc":"Applies the given function to all previous siblings of the current focus without moving the Zipper . Examples iex&gt; prev = for n &lt;- [ 1 , 2 , 3 , 4 ] , do : ExRoseTree . new ( n ) ...&gt; tree = ExRoseTree . new ( 5 ) ...&gt; z = ExRoseTree.Zipper . new ( tree , prev : prev ) ...&gt; map_fn = &amp; ExRoseTree . map_term ( &amp;1 , fn term -&gt; term * 2 end ) ...&gt; ExRoseTree.Zipper . map_previous_siblings ( z , &amp; map_fn . ( &amp;1 ) ) % ExRoseTree.Zipper { focus : % ExRoseTree { term : 5 , children : [ ] } , prev : [ % ExRoseTree { term : 8 , children : [ ] } , % ExRoseTree { term : 6 , children : [ ] } , % ExRoseTree { term : 4 , children : [ ] } , % ExRoseTree { term : 2 , children : [ ] } ] , next : [ ] , path : [ ] }","ref":"ExRoseTree.Zipper.html#map_previous_siblings/2","title":"ExRoseTree.Zipper.map_previous_siblings/2","type":"function"},{"doc":"Repeats a call to the given move_fn() , by the given number of reps . Examples iex&gt; loc_trees = for n &lt;- [ 4 , 3 , 2 , 1 ] , do : ExRoseTree . new ( n ) ...&gt; locs = for n &lt;- loc_trees , do : ExRoseTree.Zipper.Location . new ( n ) ...&gt; tree = ExRoseTree . new ( 5 ) ...&gt; z = ExRoseTree.Zipper . new ( tree , path : locs ) ...&gt; move_fn = &amp; ExRoseTree.Zipper . parent / 1 ...&gt; z = ExRoseTree.Zipper . move_for ( z , move_fn , 2 ) ...&gt; ExRoseTree.Zipper . current_focus ( z ) . term 3","ref":"ExRoseTree.Zipper.html#move_for/3","title":"ExRoseTree.Zipper.move_for/3","type":"function"},{"doc":"Moves a direction in the Zipper , determined by the move_fn() , if and only if the provided predicate function returns true when applied to the next node. Otherwise, returns nil .","ref":"ExRoseTree.Zipper.html#move_if/3","title":"ExRoseTree.Zipper.move_if/3","type":"function"},{"doc":"Continuously moves a direction in the Zipper , determined by the move_fn() , until the provided predicate function returns true when applied to the next node. Otherwise, returns nil .","ref":"ExRoseTree.Zipper.html#move_until/3","title":"ExRoseTree.Zipper.move_until/3","type":"function"},{"doc":"Repeats the given move_fn() while the given predicate remains true . If no custom predicate is given, the move_fn() will repeat until it no longer can.","ref":"ExRoseTree.Zipper.html#move_while/3","title":"ExRoseTree.Zipper.move_while/3","type":"function"},{"doc":"Returns a new Zipper with its focus on the given ExRoseTree . Optionally take a list of previous and next sibling trees when using the :prev and :next keyword options. Note that a Zipper maintains the list of previous siblings in reverse order internally, and this function performs that reversal itself, so do not pre-reverse your list of previous siblings when using that option! Examples iex&gt; tree = ExRoseTree . new ( 5 ) ...&gt; ExRoseTree.Zipper . new ( tree ) % ExRoseTree.Zipper { focus : % ExRoseTree { term : 5 , children : [ ] } , prev : [ ] , next : [ ] , path : [ ] } iex&gt; prev = for n &lt;- [ 1 , 2 , 3 , 4 ] , do : ExRoseTree . new ( n ) ...&gt; tree = ExRoseTree . new ( 5 ) ...&gt; ExRoseTree.Zipper . new ( tree , prev : prev ) % ExRoseTree.Zipper { focus : % ExRoseTree { term : 5 , children : [ ] } , prev : [ % ExRoseTree { term : 4 , children : [ ] } , % ExRoseTree { term : 3 , children : [ ] } , % ExRoseTree { term : 2 , children : [ ] } , % ExRoseTree { term : 1 , children : [ ] } ] , next : [ ] , path : [ ] } iex&gt; loc_trees = for n &lt;- [ 4 , 3 , 2 , 1 ] , do : ExRoseTree . new ( n ) ...&gt; locs = for n &lt;- loc_trees , do : ExRoseTree.Zipper.Location . new ( n ) ...&gt; tree = ExRoseTree . new ( 5 ) ...&gt; ExRoseTree.Zipper . new ( tree , path : locs ) % ExRoseTree.Zipper { focus : % ExRoseTree { term : 5 , children : [ ] } , prev : [ ] , next : [ ] , path : [ % ExRoseTree.Zipper.Location { prev : [ ] , term : 4 , next : [ ] } , % ExRoseTree.Zipper.Location { prev : [ ] , term : 3 , next : [ ] } , % ExRoseTree.Zipper.Location { prev : [ ] , term : 2 , next : [ ] } , % ExRoseTree.Zipper.Location { prev : [ ] , term : 1 , next : [ ] } ] }","ref":"ExRoseTree.Zipper.html#new/2","title":"ExRoseTree.Zipper.new/2","type":"function"},{"doc":"Builds a new ExRoseTree.Zipper.Location out of the current Zipper . Examples iex&gt; next = for n &lt;- [ 6 , 7 , 8 , 9 ] , do : ExRoseTree . new ( n ) ...&gt; tree = ExRoseTree . new ( 5 ) ...&gt; z = ExRoseTree.Zipper . new ( tree , next : next ) ...&gt; ExRoseTree.Zipper . new_location ( z ) % ExRoseTree.Zipper.Location { prev : [ ] , term : 5 , next : [ % ExRoseTree { term : 6 , children : [ ] } , % ExRoseTree { term : 7 , children : [ ] } , % ExRoseTree { term : 8 , children : [ ] } , % ExRoseTree { term : 9 , children : [ ] } ] }","ref":"ExRoseTree.Zipper.html#new_location/1","title":"ExRoseTree.Zipper.new_location/1","type":"function"},{"doc":"Recursively searches the path for the first, next &quot;ancestral&quot; pibling. That is, if a next pibling is not found for the parent, it will search the grandparent. If one is not found for the grandparent, it will search the great-grandparent. And so on, until it reaches the root. If the root is reached and it does not have a next pibling, the function returns nil .","ref":"ExRoseTree.Zipper.html#next_ancestral_pibling/2","title":"ExRoseTree.Zipper.next_ancestral_pibling/2","type":"function"},{"doc":"Recursively searches the descendant branches of the next sibling for the next &quot;descendant nibling&quot; of the current focus. That is, if a next nibling is found, it will then look for the first child of that tree (the &quot;descendant nibling&quot;). It will repeat the search if one is found, and it will continue until no more are found, returning the last one visited.","ref":"ExRoseTree.Zipper.html#next_descendant_nibling/2","title":"ExRoseTree.Zipper.next_descendant_nibling/2","type":"function"},{"doc":"Searches for the next extended cousin or the next first-cousin of the focused tree.","ref":"ExRoseTree.Zipper.html#next_extended_cousin/2","title":"ExRoseTree.Zipper.next_extended_cousin/2","type":"function"},{"doc":"Searches for the first child of the next extended cousin--aka, the next extended nibling--of the focused tree.","ref":"ExRoseTree.Zipper.html#next_extended_nibling/2","title":"ExRoseTree.Zipper.next_extended_nibling/2","type":"function"},{"doc":"Searches for the next extended cousin of the parent--aka, the next extended pibling--of the focused tree. Note: Extended Pibling here really means parent-cousin n-times removed, and is a bit of a neolism, since pibling technically means parent-sibling.","ref":"ExRoseTree.Zipper.html#next_extended_pibling/2","title":"ExRoseTree.Zipper.next_extended_pibling/2","type":"function"},{"doc":"Moves the focus to the next first-cousin -- the first child of the next pibling with children -- of the current focus. If not found, returns nil .","ref":"ExRoseTree.Zipper.html#next_first_cousin/2","title":"ExRoseTree.Zipper.next_first_cousin/2","type":"function"},{"doc":"Moves the focus to the next grand-nibling -- the first grandchild of the next sibling -- of the current focus. If not found, returns nil .","ref":"ExRoseTree.Zipper.html#next_grandnibling/2","title":"ExRoseTree.Zipper.next_grandnibling/2","type":"function"},{"doc":"Moves the focus to the next grandpibling -- the next sibling of the grandparent -- of the current focus. If not found, returns nil .","ref":"ExRoseTree.Zipper.html#next_grandpibling/2","title":"ExRoseTree.Zipper.next_grandpibling/2","type":"function"},{"doc":"Moves the focus to the next nibling -- the first child of the first next sibling with children -- before the current focus. If not found, returns nil .","ref":"ExRoseTree.Zipper.html#next_nibling/2","title":"ExRoseTree.Zipper.next_nibling/2","type":"function"},{"doc":"Moves the focus to the next pibling -- the next sibling of the parent -- of the current focus. If not found, returns nil .","ref":"ExRoseTree.Zipper.html#next_pibling/2","title":"ExRoseTree.Zipper.next_pibling/2","type":"function"},{"doc":"Moves the focus to the next second-cousin -- the first grandchild of the next grandpibling with grandchildren -- of the current focus. If not found, returns nil .","ref":"ExRoseTree.Zipper.html#next_second_cousin/2","title":"ExRoseTree.Zipper.next_second_cousin/2","type":"function"},{"doc":"Moves focus to the next sibling of the current focus. If there are no more siblings after the current focus, returns nil . Examples iex&gt; prev = for n &lt;- [ 1 , 2 , 3 , 4 ] , do : ExRoseTree . new ( n ) ...&gt; next = for n &lt;- [ 6 , 7 , 8 , 9 ] , do : ExRoseTree . new ( n ) ...&gt; tree = ExRoseTree . new ( 5 ) ...&gt; z = ExRoseTree.Zipper . new ( tree , prev : prev , next : next ) ...&gt; ExRoseTree.Zipper . next_sibling ( z ) % ExRoseTree.Zipper { focus : % ExRoseTree { term : 6 , children : [ ] } , prev : [ % ExRoseTree { term : 5 , children : [ ] } , % ExRoseTree { term : 4 , children : [ ] } , % ExRoseTree { term : 3 , children : [ ] } , % ExRoseTree { term : 2 , children : [ ] } , % ExRoseTree { term : 1 , children : [ ] } ] , next : [ % ExRoseTree { term : 7 , children : [ ] } , % ExRoseTree { term : 8 , children : [ ] } , % ExRoseTree { term : 9 , children : [ ] } ] , path : [ ] }","ref":"ExRoseTree.Zipper.html#next_sibling/2","title":"ExRoseTree.Zipper.next_sibling/2","type":"function"},{"doc":"Returns the siblings that come after the current focus. Examples iex&gt; next = for t &lt;- [ 6 , 7 , 8 , 9 ] , do : ExRoseTree . new ( t ) ...&gt; tree = ExRoseTree . new ( 5 ) ...&gt; z = ExRoseTree.Zipper . new ( tree , next : next ) ...&gt; ExRoseTree.Zipper . next_siblings ( z ) [ % ExRoseTree { term : 6 , children : [ ] } , % ExRoseTree { term : 7 , children : [ ] } , % ExRoseTree { term : 8 , children : [ ] } , % ExRoseTree { term : 9 , children : [ ] } ]","ref":"ExRoseTree.Zipper.html#next_siblings/1","title":"ExRoseTree.Zipper.next_siblings/1","type":"function"},{"doc":"Moves the focus to the parent ExRoseTree.Zipper.Location . If at the root, thus no parent, returns nil . Examples iex&gt; prev = for n &lt;- [ 3 , 4 ] , do : ExRoseTree . new ( n ) ...&gt; loc_trees = for n &lt;- [ 2 , 1 ] , do : ExRoseTree . new ( n ) ...&gt; locs = for n &lt;- loc_trees , do : ExRoseTree.Zipper.Location . new ( n ) ...&gt; tree = ExRoseTree . new ( 5 ) ...&gt; z = ExRoseTree.Zipper . new ( tree , prev : prev , path : locs ) ...&gt; ExRoseTree.Zipper . parent ( z ) % ExRoseTree.Zipper { focus : % ExRoseTree { term : 2 , children : [ % ExRoseTree { term : 3 , children : [ ] } , % ExRoseTree { term : 4 , children : [ ] } , % ExRoseTree { term : 5 , children : [ ] } ] } , prev : [ ] , next : [ ] , path : [ % ExRoseTree.Zipper.Location { prev : [ ] , term : 1 , next : [ ] } ] }","ref":"ExRoseTree.Zipper.html#parent/1","title":"ExRoseTree.Zipper.parent/1","type":"function"},{"doc":"Returns the current Zipper 's parent ExRoseTree.Zipper.Location . Examples iex&gt; loc_trees = for n &lt;- [ 4 , 3 , 2 , 1 ] , do : ExRoseTree . new ( n ) ...&gt; locs = for n &lt;- loc_trees , do : ExRoseTree.Zipper.Location . new ( n ) ...&gt; tree = ExRoseTree . new ( 5 ) ...&gt; z = ExRoseTree.Zipper . new ( tree , path : locs ) ...&gt; ExRoseTree.Zipper . parent_location ( z ) % ExRoseTree.Zipper.Location { prev : [ ] , term : 4 , next : [ ] }","ref":"ExRoseTree.Zipper.html#parent_location/1","title":"ExRoseTree.Zipper.parent_location/1","type":"function"},{"doc":"Returns the term in the current Zipper 's parent ExRoseTree.Zipper.Location . Examples iex&gt; loc_trees = for n &lt;- [ 4 , 3 , 2 , 1 ] , do : ExRoseTree . new ( n ) ...&gt; locs = for n &lt;- loc_trees , do : ExRoseTree.Zipper.Location . new ( n ) ...&gt; tree = ExRoseTree . new ( 5 ) ...&gt; z = ExRoseTree.Zipper . new ( tree , path : locs ) ...&gt; ExRoseTree.Zipper . parent_term ( z ) 4","ref":"ExRoseTree.Zipper.html#parent_term/1","title":"ExRoseTree.Zipper.parent_term/1","type":"function"},{"doc":"Moves the focus to the pibling of the current focus at the given index. If no pibling is found at that index, returns nil .","ref":"ExRoseTree.Zipper.html#pibling_at/2","title":"ExRoseTree.Zipper.pibling_at/2","type":"function"},{"doc":"Removes the first sibling from the Zipper .","ref":"ExRoseTree.Zipper.html#pop_first_sibling/1","title":"ExRoseTree.Zipper.pop_first_sibling/1","type":"function"},{"doc":"Removes the last sibling from the Zipper .","ref":"ExRoseTree.Zipper.html#pop_last_sibling/1","title":"ExRoseTree.Zipper.pop_last_sibling/1","type":"function"},{"doc":"Removes the next sibling from the Zipper .","ref":"ExRoseTree.Zipper.html#pop_next_sibling/1","title":"ExRoseTree.Zipper.pop_next_sibling/1","type":"function"},{"doc":"Removes a sibling from the Zipper 's next siblings at the given index.","ref":"ExRoseTree.Zipper.html#pop_next_sibling_at/2","title":"ExRoseTree.Zipper.pop_next_sibling_at/2","type":"function"},{"doc":"Removes the previous sibling from the Zipper .","ref":"ExRoseTree.Zipper.html#pop_previous_sibling/1","title":"ExRoseTree.Zipper.pop_previous_sibling/1","type":"function"},{"doc":"Removes a sibling from the Zipper 's prev siblings at the given index.","ref":"ExRoseTree.Zipper.html#pop_previous_sibling_at/2","title":"ExRoseTree.Zipper.pop_previous_sibling_at/2","type":"function"},{"doc":"Prepends a new sibling to the Ziper's prev siblings.","ref":"ExRoseTree.Zipper.html#prepend_first_sibling/2","title":"ExRoseTree.Zipper.prepend_first_sibling/2","type":"function"},{"doc":"Prepends a new sibling to the Zipper 's next siblings.","ref":"ExRoseTree.Zipper.html#prepend_next_sibling/2","title":"ExRoseTree.Zipper.prepend_next_sibling/2","type":"function"},{"doc":"Recursively searches the path for the first, previous &quot;ancestral&quot; pibling. That is, if a previous pibling is not found for the parent, it will search the grandparent. If one is not found for the grandparent, it will search the great-grandparent. And so on, until it reaches the root. If the root is reached and it does not have a previous pibling, the function returns nil .","ref":"ExRoseTree.Zipper.html#previous_ancestral_pibling/2","title":"ExRoseTree.Zipper.previous_ancestral_pibling/2","type":"function"},{"doc":"Recursively searches the descendant branches of the previous sibling for the previous &quot;descendant nibling&quot; of the current focus. That is, if a previous nibling is found, it will then look for the last child of that tree (the &quot;descendant nibling&quot;). It will repeat the search if one is found, and it will continue until no more are found, returning the last one visited.","ref":"ExRoseTree.Zipper.html#previous_descendant_nibling/2","title":"ExRoseTree.Zipper.previous_descendant_nibling/2","type":"function"},{"doc":"Searches for the previous extended cousin or the previous first-cousin of the focused tree.","ref":"ExRoseTree.Zipper.html#previous_extended_cousin/2","title":"ExRoseTree.Zipper.previous_extended_cousin/2","type":"function"},{"doc":"Searches for the last child of the previous extended cousin--aka, the previous extended nibling--of the focused tree.","ref":"ExRoseTree.Zipper.html#previous_extended_nibling/2","title":"ExRoseTree.Zipper.previous_extended_nibling/2","type":"function"},{"doc":"Searches for the previous extended cousin of the parent--aka, the previous extended pibling--of the focused tree. Note: Extended Pibling here really means parent-cousin n-times removed, and is a bit of a neolism, since pibling technically means parent-sibling.","ref":"ExRoseTree.Zipper.html#previous_extended_pibling/2","title":"ExRoseTree.Zipper.previous_extended_pibling/2","type":"function"},{"doc":"Moves the focus to the previous first-cousin -- the last child of the previous pibling with children -- of the current focus. If not found, returns nil .","ref":"ExRoseTree.Zipper.html#previous_first_cousin/2","title":"ExRoseTree.Zipper.previous_first_cousin/2","type":"function"},{"doc":"Moves the focus to the previous grand-nibling -- the last grandchild of the previous sibling -- of the current focus. If not found, returns nil .","ref":"ExRoseTree.Zipper.html#previous_grandnibling/2","title":"ExRoseTree.Zipper.previous_grandnibling/2","type":"function"},{"doc":"Moves the focus to the previous grandpibling -- the previous sibling of the grandparent -- of the current focus. If not found, returns nil .","ref":"ExRoseTree.Zipper.html#previous_grandpibling/2","title":"ExRoseTree.Zipper.previous_grandpibling/2","type":"function"},{"doc":"Moves the focus to the previous nibling -- the last child of the first previous sibling with children -- before the current focus. If not found, returns nil .","ref":"ExRoseTree.Zipper.html#previous_nibling/2","title":"ExRoseTree.Zipper.previous_nibling/2","type":"function"},{"doc":"Moves the focus to the previous pibling -- the previous sibling of the parent -- of the current focus. If not found, returns nil .","ref":"ExRoseTree.Zipper.html#previous_pibling/2","title":"ExRoseTree.Zipper.previous_pibling/2","type":"function"},{"doc":"Moves the focus to the previous second-cousin -- the last grandchild of the previous grandpibling with grandchildren -- of the current focus. If not found, returns nil .","ref":"ExRoseTree.Zipper.html#previous_second_cousin/2","title":"ExRoseTree.Zipper.previous_second_cousin/2","type":"function"},{"doc":"Moves focus to the previous sibling to the current focus. If there are no more siblings before the current focus, returns nil . Examples iex&gt; prev = for n &lt;- [ 1 , 2 , 3 , 4 ] , do : ExRoseTree . new ( n ) ...&gt; next = for n &lt;- [ 6 , 7 , 8 , 9 ] , do : ExRoseTree . new ( n ) ...&gt; tree = ExRoseTree . new ( 5 ) ...&gt; z = ExRoseTree.Zipper . new ( tree , prev : prev , next : next ) ...&gt; ExRoseTree.Zipper . previous_sibling ( z ) % ExRoseTree.Zipper { focus : % ExRoseTree { term : 4 , children : [ ] } , prev : [ % ExRoseTree { term : 3 , children : [ ] } , % ExRoseTree { term : 2 , children : [ ] } , % ExRoseTree { term : 1 , children : [ ] } ] , next : [ % ExRoseTree { term : 5 , children : [ ] } , % ExRoseTree { term : 6 , children : [ ] } , % ExRoseTree { term : 7 , children : [ ] } , % ExRoseTree { term : 8 , children : [ ] } , % ExRoseTree { term : 9 , children : [ ] } ] , path : [ ] }","ref":"ExRoseTree.Zipper.html#previous_sibling/2","title":"ExRoseTree.Zipper.previous_sibling/2","type":"function"},{"doc":"Returns the siblings that come before the current focus. Examples iex&gt; prev = for t &lt;- [ 1 , 2 , 3 , 4 ] , do : ExRoseTree . new ( t ) ...&gt; tree = ExRoseTree . new ( 5 ) ...&gt; z = ExRoseTree.Zipper . new ( tree , prev : prev ) ...&gt; ExRoseTree.Zipper . previous_siblings ( z ) [ % ExRoseTree { term : 1 , children : [ ] } , % ExRoseTree { term : 2 , children : [ ] } , % ExRoseTree { term : 3 , children : [ ] } , % ExRoseTree { term : 4 , children : [ ] } ]","ref":"ExRoseTree.Zipper.html#previous_siblings/1","title":"ExRoseTree.Zipper.previous_siblings/1","type":"function"},{"doc":"Removes the current focus and then moves the focus to one of three places: the next sibling, if one exists, else the previous sibling, if one exists, else the parent, if one exists If none of those conditions exist, it will return an empty Zipper . In any case, the new Zipper will be returned as the first item in a tuple, while the removed focus will be returned as the second item. Examples iex&gt; tree = ExRoseTree . new ( 5 ) ...&gt; prev_siblings = for n &lt;- [ 3 , 4 ] , do : ExRoseTree . new ( n ) ...&gt; next_siblings = for n &lt;- [ 6 , 7 ] , do : ExRoseTree . new ( n ) ...&gt; z = ExRoseTree.Zipper . new ( tree , prev : prev_siblings , next : next_siblings ) ...&gt; ExRoseTree.Zipper . remove_focus ( z ) { % ExRoseTree.Zipper { focus : % ExRoseTree { term : 6 , children : [ ] } , prev : [ % ExRoseTree { term : 4 , children : [ ] } , % ExRoseTree { term : 3 , children : [ ] } ] , next : [ % ExRoseTree { term : 7 , children : [ ] } ] , path : [ ] } , % ExRoseTree { term : 5 , children : [ ] } }","ref":"ExRoseTree.Zipper.html#remove_focus/1","title":"ExRoseTree.Zipper.remove_focus/1","type":"function"},{"doc":"Rewinds the Zipper and accumulates an additional value using the provided acc_fn() . Returns a tuple including the root Zipper and the accumulated value.","ref":"ExRoseTree.Zipper.html#rewind_accumulate/3","title":"ExRoseTree.Zipper.rewind_accumulate/3","type":"function"},{"doc":"Searches for a predicate match by rewinding the path in the Zipper . If no match is found, returns nil .","ref":"ExRoseTree.Zipper.html#rewind_find/2","title":"ExRoseTree.Zipper.rewind_find/2","type":"function"},{"doc":"Rewinds a Zipper along the path by the given number of reps .","ref":"ExRoseTree.Zipper.html#rewind_for/2","title":"ExRoseTree.Zipper.rewind_for/2","type":"function"},{"doc":"Rewinds a Zipper along the path if the provided predicate function returns true when applied to the parent node. Otherwise, returns nil .","ref":"ExRoseTree.Zipper.html#rewind_if/2","title":"ExRoseTree.Zipper.rewind_if/2","type":"function"},{"doc":"Rewinds the Zipper and maps the term at each node using the provided map_fn() . Returns the new Zipper at the root with mapped values.","ref":"ExRoseTree.Zipper.html#rewind_map/2","title":"ExRoseTree.Zipper.rewind_map/2","type":"function"},{"doc":"Rewinds a Zipper back to the root. Examples iex&gt; loc_trees = for n &lt;- [ 4 , 3 , 2 , 1 ] , do : ExRoseTree . new ( n ) ...&gt; locs = for n &lt;- loc_trees , do : ExRoseTree.Zipper.Location . new ( n ) ...&gt; tree = ExRoseTree . new ( 5 ) ...&gt; z = ExRoseTree.Zipper . new ( tree , path : locs ) ...&gt; z = ExRoseTree.Zipper . rewind_to_root ( z ) ...&gt; ExRoseTree.Zipper . root? ( z ) true","ref":"ExRoseTree.Zipper.html#rewind_to_root/1","title":"ExRoseTree.Zipper.rewind_to_root/1","type":"function"},{"doc":"Rewinds a Zipper continuously until the provided predicate function returns true when applied to the next parent node. Otherwise, returns nil .","ref":"ExRoseTree.Zipper.html#rewind_until/2","title":"ExRoseTree.Zipper.rewind_until/2","type":"function"},{"doc":"Rewinds a Zipper while the given predicate remains true . If no custom predicate is given, parent/1 will repeat until it reaches the root.","ref":"ExRoseTree.Zipper.html#rewind_while/2","title":"ExRoseTree.Zipper.rewind_while/2","type":"function"},{"doc":"Descend the right-most edge until it can go no further or until the optional predicate matches. Does not include siblings of starting focus.","ref":"ExRoseTree.Zipper.html#rightmost_descendant/2","title":"ExRoseTree.Zipper.rightmost_descendant/2","type":"function"},{"doc":"Returns whether or not the current Zipper is at the root of the tree. A Zipper is considered at the root when it has no ExRoseTree.Zipper.Location s in its path. Examples iex&gt; tree = ExRoseTree . new ( 5 ) ...&gt; z = ExRoseTree.Zipper . new ( tree ) ...&gt; ExRoseTree.Zipper . root? ( z ) true","ref":"ExRoseTree.Zipper.html#root?/1","title":"ExRoseTree.Zipper.root?/1","type":"function"},{"doc":"Sets the current focus of the Zipper to the given ExRoseTree . Examples iex&gt; tree = ExRoseTree . new ( 5 ) ...&gt; z = ExRoseTree.Zipper . empty ( ) ...&gt; ExRoseTree.Zipper . set_focus ( z , tree ) % ExRoseTree.Zipper { focus : % ExRoseTree { term : 5 , children : [ ] } , prev : [ ] , next : [ ] , path : [ ] }","ref":"ExRoseTree.Zipper.html#set_focus/2","title":"ExRoseTree.Zipper.set_focus/2","type":"function"},{"doc":"Sets the focused children to the given list of rose trees. A shortcut to ExRoseTree.set_children/2 and set_focus/2 . Examples iex&gt; tree = ExRoseTree . new ( 5 , [ 1 , 2 , 3 , 4 ] ) ...&gt; z = ExRoseTree.Zipper . new ( tree ) ...&gt; new_children = for t &lt;- [ 6 , 7 , 8 , 9 ] , do : ExRoseTree . new ( t ) ...&gt; ExRoseTree.Zipper . set_focused_children ( z , new_children ) % ExRoseTree.Zipper { focus : % ExRoseTree { term : 5 , children : [ % ExRoseTree { term : 6 , children : [ ] } , % ExRoseTree { term : 7 , children : [ ] } , % ExRoseTree { term : 8 , children : [ ] } , % ExRoseTree { term : 9 , children : [ ] } ] } , prev : [ ] , next : [ ] , path : [ ] }","ref":"ExRoseTree.Zipper.html#set_focused_children/2","title":"ExRoseTree.Zipper.set_focused_children/2","type":"function"},{"doc":"Sets the term of the current focus of the Zipper . A shortcut to using ExRoseTree.set_term/2 with set_focus/2 . Examples iex&gt; tree = ExRoseTree . new ( 5 ) ...&gt; z = ExRoseTree.Zipper . new ( tree ) ...&gt; ExRoseTree.Zipper . set_focused_term ( z , 10 ) % ExRoseTree.Zipper { focus : % ExRoseTree { term : 10 , children : [ ] } , prev : [ ] , next : [ ] , path : [ ] }","ref":"ExRoseTree.Zipper.html#set_focused_term/2","title":"ExRoseTree.Zipper.set_focused_term/2","type":"function"},{"doc":"Moves focus to the sibling of the current focus at the given index. If no sibling is found at that index, or if the provided index is the index for the current focus, returns nil . Examples iex&gt; prev = for n &lt;- [ 1 , 2 , 3 , 4 ] , do : ExRoseTree . new ( n ) ...&gt; next = for n &lt;- [ 6 , 7 , 8 , 9 ] , do : ExRoseTree . new ( n ) ...&gt; tree = ExRoseTree . new ( 5 ) ...&gt; z = ExRoseTree.Zipper . new ( tree , prev : prev , next : next ) ...&gt; ExRoseTree.Zipper . sibling_at ( z , 2 ) % ExRoseTree.Zipper { focus : % ExRoseTree { term : 3 , children : [ ] } , prev : [ % ExRoseTree { term : 2 , children : [ ] } , % ExRoseTree { term : 1 , children : [ ] } ] , next : [ % ExRoseTree { term : 4 , children : [ ] } , % ExRoseTree { term : 5 , children : [ ] } , % ExRoseTree { term : 6 , children : [ ] } , % ExRoseTree { term : 7 , children : [ ] } , % ExRoseTree { term : 8 , children : [ ] } , % ExRoseTree { term : 9 , children : [ ] } ] , path : [ ] }","ref":"ExRoseTree.Zipper.html#sibling_at/2","title":"ExRoseTree.Zipper.sibling_at/2","type":"function"},{"doc":"Moves a Zipper back to the root and returns a 2-tuple containing first, a list containing the previous siblings to the original root and second, a list containing the original root followed by its next siblings. Examples iex&gt; tree = ExRoseTree . new ( 3 , [ 6 ] ) ...&gt; prev_trees = for t &lt;- [ 1 , 2 ] , do : ExRoseTree . new ( t ) ...&gt; next_trees = for t &lt;- [ 4 , 5 ] , do : ExRoseTree . new ( t ) ...&gt; z = ExRoseTree.Zipper . new ( tree , prev : prev_trees , next : next_trees ) ...&gt; z = Zipper . last_child ( z ) ...&gt; Zipper . to_forest ( z ) { [ % ExRoseTree { term : 1 , children : [ ] } , % ExRoseTree { term : 2 , children : [ ] } ] , [ % ExRoseTree { term : 3 , children : [ % ExRoseTree { term : 6 , children : [ ] } ] } , % ExRoseTree { term : 4 , children : [ ] } , % ExRoseTree { term : 5 , children : [ ] } ] }","ref":"ExRoseTree.Zipper.html#to_forest/1","title":"ExRoseTree.Zipper.to_forest/1","type":"function"},{"doc":"Moves a Zipper back to the root and returns the current focus, the root ExRoseTree . Examples iex&gt; tree = ExRoseTree . new ( 5 , [ 1 , 2 , 3 , 4 ] ) ...&gt; z = ExRoseTree.Zipper . new ( tree ) ...&gt; z = Zipper . last_child ( z ) ...&gt; Zipper . to_tree ( z ) % ExRoseTree { term : 5 , children : [ % ExRoseTree { term : 1 , children : [ ] } , % ExRoseTree { term : 2 , children : [ ] } , % ExRoseTree { term : 3 , children : [ ] } , % ExRoseTree { term : 4 , children : [ ] } , ] }","ref":"ExRoseTree.Zipper.html#to_tree/1","title":"ExRoseTree.Zipper.to_tree/1","type":"function"},{"doc":"","ref":"ExRoseTree.Zipper.html#zipper?/1","title":"ExRoseTree.Zipper.zipper?/1","type":"macro"},{"doc":"","ref":"ExRoseTree.Zipper.html#t:acc_fn/0","title":"ExRoseTree.Zipper.acc_fn/0","type":"type"},{"doc":"","ref":"ExRoseTree.Zipper.html#t:map_fn/0","title":"ExRoseTree.Zipper.map_fn/0","type":"type"},{"doc":"","ref":"ExRoseTree.Zipper.html#t:move_fn/0","title":"ExRoseTree.Zipper.move_fn/0","type":"type"},{"doc":"","ref":"ExRoseTree.Zipper.html#t:predicate/0","title":"ExRoseTree.Zipper.predicate/0","type":"type"},{"doc":"The Zipper struct represents a contextual position within an ExRoseTree . It includes the following important pieces: focus - the current focus or context within the ExRoseTree.Zipper . Its type is that of an ExRoseTree . prev - all siblings occurring before the current focus . It's type is a list of ExRoseTree s and is maintained in reverse order, so that the immediately previous sibling to the focus is at the head of the list. next - all siblings occurring after the current focus . It's type is a list of ExRoseTree s and is maintained in standard order. path - all direct ancestors of the the current focus back to the root node. It's type is a list of ExRoseTree.Zipper.Location s and is maintained in standard order. If the path is an empty list, then the Zipper is focused at the root node.","ref":"ExRoseTree.Zipper.html#t:t/0","title":"ExRoseTree.Zipper.t/0","type":"type"},{"doc":"A Location in the path from the root of the ExRoseTree.Zipper to its current context.","ref":"ExRoseTree.Zipper.Location.html","title":"ExRoseTree.Zipper.Location","type":"module"},{"doc":"Returns whether a list of values are all Location s or not. Will return true if passed an empty list. Examples iex&gt; locs = for loc &lt;- [ 5 , 4 , 3 , 2 , 1 ] , do : ExRoseTree.Zipper.Location . new ( loc ) ...&gt; ExRoseTree.Zipper.Location . all_locations? ( locs ) true","ref":"ExRoseTree.Zipper.Location.html#all_locations?/1","title":"ExRoseTree.Zipper.Location.all_locations?/1","type":"function"},{"doc":"Returns the index of the Location in relation to its siblings. Examples iex&gt; trees = for t &lt;- [ 5 , 4 , 3 , 2 , 1 ] , do : ExRoseTree . new ( t ) ...&gt; loc = ExRoseTree.Zipper.Location . new ( 6 , prev : trees , next : [ ] ) ...&gt; ExRoseTree.Zipper.Location . index_of_term ( loc ) 5","ref":"ExRoseTree.Zipper.Location.html#index_of_term/1","title":"ExRoseTree.Zipper.Location.index_of_term/1","type":"function"},{"doc":"","ref":"ExRoseTree.Zipper.Location.html#location?/1","title":"ExRoseTree.Zipper.Location.location?/1","type":"macro"},{"doc":"Applies the given function to the Location 's term field. Examples iex&gt; loc = ExRoseTree.Zipper.Location . new ( 5 , prev : [ ] , next : [ ] ) ...&gt; ExRoseTree.Zipper.Location . map_term ( loc , &amp; ( &amp;1 * 2 ) ) % ExRoseTree.Zipper.Location { prev : [ ] , term : 10 , next : [ ] }","ref":"ExRoseTree.Zipper.Location.html#map_term/2","title":"ExRoseTree.Zipper.Location.map_term/2","type":"function"},{"doc":"Builds a new Location given a term() or an ExRoseTree as the first argument, and optional :prev and :next keywords of lists of ExRoseTree s. If the first argument is an ExRoseTree , it will unwrap its term element. Examples iex&gt; ExRoseTree.Zipper.Location . new ( 5 , prev : [ ] , next : [ ] ) % ExRoseTree.Zipper.Location { prev : [ ] , term : 5 , next : [ ] } iex&gt; tree = ExRoseTree . new ( 4 ) ...&gt; ExRoseTree.Zipper.Location . new ( 5 , prev : [ tree ] , next : [ ] ) % ExRoseTree.Zipper.Location { prev : [ % ExRoseTree { term : 4 , children : [ ] } ] , term : 5 , next : [ ] }","ref":"ExRoseTree.Zipper.Location.html#new/2","title":"ExRoseTree.Zipper.Location.new/2","type":"function"},{"doc":"A Location is made up of the term of an ExRoseTree with lists of prev and next siblings. term is an ExRoseTree term . prev is a list of ExRoseTree s. They are the siblings that occur prior the term . It is reversed such that the head of the list is the nearest previous sibling. next is a list of ExRoseTree s. They are the siblings that occur after the term .","ref":"ExRoseTree.Zipper.Location.html#t:t/0","title":"ExRoseTree.Zipper.Location.t/0","type":"type"},{"doc":"Various utility functions.","ref":"ExRoseTree.Util.html","title":"ExRoseTree.Util","type":"module"},{"doc":"A function that always returns true, regardless of what is passed to it. Examples iex&gt; ExRoseTree.Util . always ( 5 ) true iex&gt; ExRoseTree.Util . always ( false ) true","ref":"ExRoseTree.Util.html#always/1","title":"ExRoseTree.Util.always/1","type":"function"},{"doc":"Given a term and a list of potential functions to apply to the term, will return with the result of the first one that succeeds when applied to the subject term. If no functions succeed, returns nil. The list of functions should each be of type ExRoseTree.Util.result_fn() , in other words, they should return a ExRoseTree.Util.result() type. Examples iex&gt; funs = for n &lt;- [ 4 , 3 , 2 , 1 ] , do : &amp; ( if n == &amp;1 do { :ok , n } else :error end ) ...&gt; ExRoseTree.Util . first_of ( 3 , funs ) 3 iex&gt; funs = for n &lt;- [ 4 , 3 , 2 , 1 ] , do : &amp; ( if n == &amp;1 do { :ok , n } else :error end ) ...&gt; ExRoseTree.Util . first_of ( 6 , funs ) nil","ref":"ExRoseTree.Util.html#first_of/2","title":"ExRoseTree.Util.first_of/2","type":"function"},{"doc":"Given a term, a list of potential functions to apply to the term, and a list of arguments to apply to each function, will return with the result of the first one that succeeds when applied to the subject term. If no functions succeed, returns nil. The list of functions should each be of type ExRoseTree.Util.result_fn() , in other words, they should return a ExRoseTree.Util.result() type. Examples iex&gt; fun = fn x , y , add_by , sub_by -&gt; ...&gt; if x == y do { :ok , x + add_by - sub_by } else :error end ...&gt; end ...&gt; funs = for n &lt;- [ 4 , 3 , 2 , 1 ] , do : &amp; fun . ( n , &amp;1 , &amp;2 , &amp;3 ) ...&gt; ExRoseTree.Util . first_of_with_args ( 3 , funs , [ 2 , 1 ] ) 4","ref":"ExRoseTree.Util.html#first_of_with_args/3","title":"ExRoseTree.Util.first_of_with_args/3","type":"function"},{"doc":"Given a term, a list of potential functions to apply to the term, and a keyword list of options to apply to each function, will return with the result of the first one that succeeds when applied to the subject term. If no functions succeed, returns nil. The list of functions should each be of type ExRoseTree.Util.result_fn() , in other words, they should return a ExRoseTree.Util.result() type. Examples iex&gt; fun = fn x , y , z -&gt; ...&gt; mult_by = Keyword . get ( z , :mult_by , 1 ) ...&gt; if x == y do { :ok , x * mult_by } else :error end ...&gt; end ...&gt; funs = for n &lt;- [ 4 , 3 , 2 , 1 ] , do : &amp; fun . ( n , &amp;1 , &amp;2 ) ...&gt; ExRoseTree.Util . first_of_with_opts ( 3 , funs , [ mult_by : 2 ] ) 6","ref":"ExRoseTree.Util.html#first_of_with_opts/3","title":"ExRoseTree.Util.first_of_with_opts/3","type":"function"},{"doc":"A function that applies a predicate to a term. If the function application is true, returns the original term. If false, returns nil. Examples iex&gt; ExRoseTree.Util . maybe ( 5 , &amp; ( &amp;1 == 5 ) ) 5 iex&gt; ExRoseTree.Util . maybe ( 5 , &amp; ( &amp;1 == 1 ) ) nil","ref":"ExRoseTree.Util.html#maybe/2","title":"ExRoseTree.Util.maybe/2","type":"function"},{"doc":"A function that always returns false, regardless of what is passed to it. Examples iex&gt; ExRoseTree.Util . never ( 5 ) false iex&gt; ExRoseTree.Util . never ( true ) false","ref":"ExRoseTree.Util.html#never/1","title":"ExRoseTree.Util.never/1","type":"function"},{"doc":"Similar to Enum.split/2 but with specialized behavior. It is optimized to return the list of elements that come before the index in reverse order. This is ideal for the context-aware nature of Zippers. Also unlike Enum.split/2 , if given an index that is greater than or equal to the total elements in the given list or if given a negative index, this function will not perform a split, and will return two empty lists. Examples iex&gt; ExRoseTree.Util . split_at ( [ 1 , 2 , 3 , 4 , 5 ] , 2 ) { [ 2 , 1 ] , [ 3 , 4 , 5 ] } iex&gt; ExRoseTree.Util . split_at ( [ 1 , 2 , 3 , 4 , 5 ] , 10 ) { [ ] , [ ] }","ref":"ExRoseTree.Util.html#split_at/2","title":"ExRoseTree.Util.split_at/2","type":"function"},{"doc":"Similar to Enum.split/2 , Enum.split_while/2 , and Enum.split_with/2 , split_when/2 instead takes a list of elements and a predicate to apply to each element. The first element that passes the predicate is where the list of elements will be split, with the target element as the head of the second list in the return value. Like with split_at/2 , the first list of elements are returned in reverse order. Examples iex&gt; ExRoseTree.Util . split_when ( [ 1 , 2 , 3 , 4 , 5 ] , fn x -&gt; x == 3 end ) { [ 2 , 1 ] , [ 3 , 4 , 5 ] }","ref":"ExRoseTree.Util.html#split_when/2","title":"ExRoseTree.Util.split_when/2","type":"function"},{"doc":"","ref":"ExRoseTree.Util.html#t:result/0","title":"ExRoseTree.Util.result/0","type":"type"},{"doc":"","ref":"ExRoseTree.Util.html#t:result_fn/0","title":"ExRoseTree.Util.result_fn/0","type":"type"},{"doc":"","ref":"ExRoseTree.Support.Generators.html","title":"ExRoseTree.Support.Generators","type":"module"},{"doc":"","ref":"ExRoseTree.Support.Generators.html#add_zipper_locations/2","title":"ExRoseTree.Support.Generators.add_zipper_locations/2","type":"function"},{"doc":"","ref":"ExRoseTree.Support.Generators.html#add_zipper_siblings/2","title":"ExRoseTree.Support.Generators.add_zipper_siblings/2","type":"function"},{"doc":"","ref":"ExRoseTree.Support.Generators.html#allot_remaining_shares/2","title":"ExRoseTree.Support.Generators.allot_remaining_shares/2","type":"function"},{"doc":"","ref":"ExRoseTree.Support.Generators.html#default_init/1","title":"ExRoseTree.Support.Generators.default_init/1","type":"function"},{"doc":"","ref":"ExRoseTree.Support.Generators.html#default_unfolder/2","title":"ExRoseTree.Support.Generators.default_unfolder/2","type":"function"},{"doc":"","ref":"ExRoseTree.Support.Generators.html#new_seed/3","title":"ExRoseTree.Support.Generators.new_seed/3","type":"function"},{"doc":"","ref":"ExRoseTree.Support.Generators.html#random_number_of_nodes/0","title":"ExRoseTree.Support.Generators.random_number_of_nodes/0","type":"function"},{"doc":"","ref":"ExRoseTree.Support.Generators.html#random_tree/1","title":"ExRoseTree.Support.Generators.random_tree/1","type":"function"},{"doc":"","ref":"ExRoseTree.Support.Generators.html#random_zipper/1","title":"ExRoseTree.Support.Generators.random_zipper/1","type":"function"},{"doc":"Sample ExRoseTrees for use in development and testing.","ref":"ExRoseTree.Support.Trees.html","title":"ExRoseTree.Support.Trees","type":"module"},{"doc":"","ref":"ExRoseTree.Support.Trees.html#empty_tree/0","title":"ExRoseTree.Support.Trees.empty_tree/0","type":"function"},{"doc":"","ref":"ExRoseTree.Support.Trees.html#leaf_tree/0","title":"ExRoseTree.Support.Trees.leaf_tree/0","type":"function"},{"doc":"","ref":"ExRoseTree.Support.Trees.html#simple_tree/0","title":"ExRoseTree.Support.Trees.simple_tree/0","type":"function"},{"doc":"Sample Zippers for use in development and testing. WARNING :: Do not make modifications to the values or structures of these samples without being prepared to rewrite any unit test that relies on them!","ref":"ExRoseTree.Support.Zippers.html","title":"ExRoseTree.Support.Zippers","type":"module"},{"doc":"","ref":"ExRoseTree.Support.Zippers.html#empty_z/0","title":"ExRoseTree.Support.Zippers.empty_z/0","type":"function"},{"doc":"","ref":"ExRoseTree.Support.Zippers.html#leaf_z/0","title":"ExRoseTree.Support.Zippers.leaf_z/0","type":"function"},{"doc":"","ref":"ExRoseTree.Support.Zippers.html#simple_z/0","title":"ExRoseTree.Support.Zippers.simple_z/0","type":"function"},{"doc":"","ref":"ExRoseTree.Support.Zippers.html#z_breadth_first/0","title":"ExRoseTree.Support.Zippers.z_breadth_first/0","type":"function"},{"doc":"","ref":"ExRoseTree.Support.Zippers.html#z_breadth_first_siblings/0","title":"ExRoseTree.Support.Zippers.z_breadth_first_siblings/0","type":"function"},{"doc":"","ref":"ExRoseTree.Support.Zippers.html#z_depth_first/0","title":"ExRoseTree.Support.Zippers.z_depth_first/0","type":"function"},{"doc":"","ref":"ExRoseTree.Support.Zippers.html#z_depth_first_siblings/0","title":"ExRoseTree.Support.Zippers.z_depth_first_siblings/0","type":"function"},{"doc":"","ref":"ExRoseTree.Support.Zippers.html#z_with_1st_cousins/0","title":"ExRoseTree.Support.Zippers.z_with_1st_cousins/0","type":"function"},{"doc":"","ref":"ExRoseTree.Support.Zippers.html#z_with_2nd_cousins/0","title":"ExRoseTree.Support.Zippers.z_with_2nd_cousins/0","type":"function"},{"doc":"","ref":"ExRoseTree.Support.Zippers.html#z_with_ancestral_piblings/0","title":"ExRoseTree.Support.Zippers.z_with_ancestral_piblings/0","type":"function"},{"doc":"","ref":"ExRoseTree.Support.Zippers.html#z_with_descendant_niblings/0","title":"ExRoseTree.Support.Zippers.z_with_descendant_niblings/0","type":"function"},{"doc":"","ref":"ExRoseTree.Support.Zippers.html#z_with_extended_cousins/0","title":"ExRoseTree.Support.Zippers.z_with_extended_cousins/0","type":"function"},{"doc":"","ref":"ExRoseTree.Support.Zippers.html#z_with_extended_cousins_2/0","title":"ExRoseTree.Support.Zippers.z_with_extended_cousins_2/0","type":"function"},{"doc":"","ref":"ExRoseTree.Support.Zippers.html#z_with_extended_niblings/0","title":"ExRoseTree.Support.Zippers.z_with_extended_niblings/0","type":"function"},{"doc":"","ref":"ExRoseTree.Support.Zippers.html#z_with_grand_niblings/0","title":"ExRoseTree.Support.Zippers.z_with_grand_niblings/0","type":"function"},{"doc":"","ref":"ExRoseTree.Support.Zippers.html#z_with_grandchildren/0","title":"ExRoseTree.Support.Zippers.z_with_grandchildren/0","type":"function"},{"doc":"","ref":"ExRoseTree.Support.Zippers.html#z_with_grandchildren_2/0","title":"ExRoseTree.Support.Zippers.z_with_grandchildren_2/0","type":"function"},{"doc":"","ref":"ExRoseTree.Support.Zippers.html#z_with_grandparent/0","title":"ExRoseTree.Support.Zippers.z_with_grandparent/0","type":"function"},{"doc":"","ref":"ExRoseTree.Support.Zippers.html#z_with_grandpiblings/0","title":"ExRoseTree.Support.Zippers.z_with_grandpiblings/0","type":"function"},{"doc":"","ref":"ExRoseTree.Support.Zippers.html#z_with_great_grandchildren/0","title":"ExRoseTree.Support.Zippers.z_with_great_grandchildren/0","type":"function"},{"doc":"","ref":"ExRoseTree.Support.Zippers.html#z_with_great_grandchildren_2/0","title":"ExRoseTree.Support.Zippers.z_with_great_grandchildren_2/0","type":"function"},{"doc":"","ref":"ExRoseTree.Support.Zippers.html#z_with_great_grandparent/0","title":"ExRoseTree.Support.Zippers.z_with_great_grandparent/0","type":"function"},{"doc":"","ref":"ExRoseTree.Support.Zippers.html#z_with_niblings/0","title":"ExRoseTree.Support.Zippers.z_with_niblings/0","type":"function"},{"doc":"","ref":"ExRoseTree.Support.Zippers.html#z_with_no_ancestral_piblings/0","title":"ExRoseTree.Support.Zippers.z_with_no_ancestral_piblings/0","type":"function"},{"doc":"","ref":"ExRoseTree.Support.Zippers.html#z_with_parent/0","title":"ExRoseTree.Support.Zippers.z_with_parent/0","type":"function"},{"doc":"","ref":"ExRoseTree.Support.Zippers.html#z_with_piblings/0","title":"ExRoseTree.Support.Zippers.z_with_piblings/0","type":"function"},{"doc":"","ref":"ExRoseTree.Support.Zippers.html#z_with_siblings/0","title":"ExRoseTree.Support.Zippers.z_with_siblings/0","type":"function"}]